
library(dplyr)
library(ggplot2)
library(patchwork)
library(tikzDevice)
theme_set(theme_classic((size=18)))


calculating.IT=function(nu,beta,gamma,mu,delta.v,delta,epsilon){
  Q=beta/(gamma+mu)
  A=(delta*gamma)/(delta+mu)
  B=(delta.v*nu)/(delta.v+mu)
  C=mu
  D=beta
  E=mu+nu
  G=epsilon*beta
  H=mu+nu-((delta.v*nu)/(delta.v+mu))
  
  M=epsilon*A*D-((G*D)/Q)
  N=G*C+epsilon*A*E-((G*E)/Q)-((D*H)/Q)
  P=C*H+epsilon*B*C-((H*E)/Q)
  root.1=((-N+sqrt(N^2-(4*M*P)))/(2*M))
  root.2=((-N-sqrt(N^2-(4*M*P)))/(2*M))
  IT.value=max(root.1,root.2)
  if (IT.value<0){
    IT.value=0
  }
  return(IT.value)
}

computing.total.IT=function(nu.total,beta.p,beta.d,delta.v,delta,epsilon,phiBA){
  mu=1/(50*365)
  gamma=1/5
  n.length=301
  f=seq(from=0,to=1,length.out=n.length)
  nu.p=nu.total*(1-f)
  nu.d=nu.total*(1/phiBA)*f
  IT.p=sapply(nu.p,FUN=calculating.IT,beta=beta.p,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.d=sapply(nu.d,FUN=calculating.IT,beta=beta.d,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.sum=IT.p+(phiBA*IT.d)
  IT.avg=IT.sum/(1+phiBA)
  return(IT.avg)
}

compute.panel=function(beta.p,beta.d,delta,delta.v,epsilon,beta.type,immunity.type,phiBA){
  nu2=0.002
  nu2.IT.vector=computing.total.IT(nu.total=nu2,beta.p=beta.p,
                                   beta.d=beta.d,delta.v=delta.v,
                                   delta=delta,epsilon=epsilon,phiBA=phiBA)
  nu3=0.003
  nu3.IT.vector=computing.total.IT(nu.total=nu3,beta.p=beta.p,
                                   beta.d=beta.d,delta.v=delta.v,
                                   delta=delta,epsilon=epsilon,phiBA=phiBA)
  nu4=0.004
  nu4.IT.vector=computing.total.IT(nu.total=nu4,beta.p=beta.p,
                                   beta.d=beta.d,delta.v=delta.v,
                                   delta=delta,epsilon=epsilon,phiBA=phiBA)
  nu5=0.005
  nu5.IT.vector=computing.total.IT(nu.total=nu5,beta.p=beta.p,
                                   beta.d=beta.d,delta.v=delta.v,
                                   delta=delta,epsilon=epsilon,phiBA=phiBA)
  nu.vectors=c(rep(nu2,length(nu2.IT.vector)),
               rep(nu3,length(nu3.IT.vector)),
               rep(nu4,length(nu4.IT.vector)),
               rep(nu5,length(nu5.IT.vector)))
  fractions=c(seq(from=0,to=1,length.out=length(nu2.IT.vector)),
              seq(from=0,to=1,length.out=length(nu2.IT.vector)),
              seq(from=0,to=1,length.out=length(nu2.IT.vector)),
              seq(from=0,to=1,length.out=length(nu2.IT.vector)))
  IT.values=c(nu2.IT.vector,nu3.IT.vector,
              nu4.IT.vector,nu5.IT.vector)
  immunity.type.vector=rep(immunity.type,length(nu.vectors))
  beta.type.vector=rep(beta.type,length(nu.vectors))
  output=data.frame("nu"=nu.vectors,"f"=fractions,"IT"=IT.values,
                    "immunity"=as.character(immunity.type.vector),
                    "beta.type"=as.character(beta.type.vector),stringsAsFactors=FALSE)
  return(output)
}


make.plots=function(beta.base,phiBA){
  delta=1/(0.8*365)
  delta.v=1/(0.8*365)
  epsilon=0.8
  beta.p=beta.base
  beta.d=beta.base
  panel.02=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="symmetric",
                         immunity.type="poor\nimmunity",phiBA=phiBA)
  
  beta.p=0.8*beta.base
  beta.d=beta.base
  panel.01=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in HAR",
                         immunity.type="poor\nimmunity",phiBA=phiBA)
  
  beta.p=beta.base
  beta.d=0.8*beta.base
  panel.03=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in LAR",
                         immunity.type="poor\nimmunity",phiBA=phiBA)
  
  
  delta=1/(1*365)
  delta.v=1/(1*365)
  epsilon=0.7
  beta.p=beta.base
  beta.d=beta.base
  panel.12=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="symmetric",
                         immunity.type="intermediate\nimmunity",phiBA=phiBA)
  
  beta.p=0.8*beta.base
  beta.d=beta.base
  panel.11=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in HAR",
                         immunity.type="intermediate\nimmunity",phiBA=phiBA)
  
  beta.p=beta.base
  beta.d=0.8*beta.base
  panel.13=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in LAR",
                         immunity.type="intermediate\nimmunity",phiBA=phiBA)
  
  delta=1/(1.5*365)
  delta.v=1/(1.5*365)
  epsilon=0.6
  beta.p=beta.base
  beta.d=beta.base
  panel.22=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="symmetric",
                         immunity.type="good\nimmunity",phiBA=phiBA)
  
  beta.p=0.8*beta.base
  beta.d=beta.base
  panel.21=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in HAR",
                         immunity.type="good\nimmunity",phiBA=phiBA)
  
  beta.p=beta.base
  beta.d=0.8*beta.base
  panel.23=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in LAR",
                         immunity.type="good\nimmunity",phiBA=phiBA)
  
  delta=1/(2*365)
  delta.v=1/(2*365)
  epsilon=0.5
  beta.p=beta.base
  beta.d=beta.base
  panel.32=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="symmetric",
                         immunity.type="robust\nimmunity",phiBA=phiBA)
  
  beta.p=0.8*beta.base
  beta.d=beta.base
  panel.31=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in HAR",
                         immunity.type="robust\nimmunity",phiBA=phiBA)
  
  beta.p=beta.base
  beta.d=0.8*beta.base
  panel.33=compute.panel(beta.p=beta.p,beta.d=beta.d,delta=delta,epsilon=epsilon,
                         delta.v=delta.v,beta.type="less transmission in LAR",
                         immunity.type="robust\nimmunity",phiBA=phiBA)
  
  bound.data=bind_rows(panel.01,panel.02,panel.03,
                       panel.11,panel.12,panel.13,
                       panel.21,panel.22,panel.23,
                       panel.31,panel.32,panel.33,.id=NULL)
  bound.data$nu=as.character(bound.data$nu)
  bound.data$immunity=factor(bound.data$immunity,
                             levels=c("poor\nimmunity","intermediate\nimmunity",
                                      "good\nimmunity","robust\nimmunity"))
  bound.data$beta.type=factor(bound.data$beta.type,
                              levels=c("symmetric","less transmission in HAR",
                                       "less transmission in LAR"))
  if (phiBA==1){
    y.axis.label.value="Average infections at equilibrium across both countries"
  } else {
    y.axis.label.value="Weighted average infections at equilibrium across both countries"
  }
  f=ggplot(data=bound.data)+geom_line(aes(x=f,y=IT,color=nu),size=0.8)+facet_grid(beta.type~immunity)+
    theme(panel.spacing=unit(1.5,"lines"))+theme_bw()+scale_x_continuous(breaks=c(0,1))+
    xlab("Fraction $f$ of vaccines allocated by HAR to LAR")+ylab(y.axis.label.value)+
    scale_color_discrete(name="Global\nvaccination\nrate $\\nu_{\\rm tot}$\n(per day)")
  return(f)
}

computing.IT.separate=function(nu.total,beta.p,beta.d,delta.v,delta,epsilon,phiBA){
  mu=1/(50*365)
  gamma=1/5
  n.length=301
  f=seq(from=0,to=1,length.out=n.length)
  nu.p=nu.total*(1-f)
  nu.d=nu.total*(1/phiBA)*f
  IT.p=sapply(nu.p,FUN=calculating.IT,beta=beta.p,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.d=sapply(nu.d,FUN=calculating.IT,beta=beta.d,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.separate=list(IT.p,IT.d,f)
  return(IT.separate)
}

plot.underlying=function(list.in){
  values=c(list.in[[1]],list.in[[2]])
  country=c(rep("HAR",length(list.in[[1]])),rep("LAR",length(list.in[[2]])))
  for.plot=data.frame(values,country,f=c(list.in[[3]],list.in[[3]]))
  plot.out=ggplot(data=for.plot)+geom_line(aes(x=f,y=values,linetype=country),size=0.8)+
    scale_y_continuous(limits=c(0,0.01))+
    theme(text=element_text(size=12))+
    ylab("Equilibrium infections")+
    xlab("Fraction $f$ of vaccines allocated by HAR to LAR")+
    theme(legend.position=c(0.5,0.9))+scale_linetype(name=NULL)
  return(plot.out)
}

combined.figure=function(phiBA){
  reg.beta=make.plots(beta.base=2.3/5,phiBA=phiBA)
  higher.beta=make.plots(beta.base=1.3*2.3/5,phiBA=phiBA)
  
  reg.beta.title=reg.beta+
    ggtitle(label="Baseline transmission, total infections")+
    theme(plot.title=element_text(hjust=0.5,face="bold",size=12))
  higher.beta.title=higher.beta+
    ggtitle(label="Overall higher transmission, total infections")+
    theme(plot.title=element_text(hjust=0.5,face="bold",size=12))
  
  combined=((reg.beta.title|higher.beta.title))+plot_annotation(tag_levels="A")&
    theme(plot.tag=element_text(size=22,face='bold'))
  
  
  beta.base=2.3/5
  delta=1/(0.8*365)
  delta.v=1/(0.8*365)
  epsilon=0.8
  beta.p=beta.base
  beta.d=beta.base
  nu.value=0.004
  IT.vectors.separate.1=computing.IT.separate(nu.total=nu.value,beta.p=beta.p,
                                              beta.d=beta.d,delta.v=delta.v,
                                              delta=delta,epsilon=epsilon,phiBA=phiBA)
  plot1=plot.underlying(IT.vectors.separate.1)
  plot1.title=plot1+ggtitle(label="symmetric, global vaccination\nrate $\\nu_{\\rm tot}=0.004$ (per day)")+
    theme(plot.title=element_text(hjust=0.5,face="bold",size=12))
  beta.p.modif=0.8*beta.base
  IT.vectors.separate.2=computing.IT.separate(nu.total=nu.value,beta.p=beta.p.modif,
                                              beta.d=beta.d,delta.v=delta.v,
                                              delta=delta,epsilon=epsilon,phiBA=phiBA)
  plot2=plot.underlying(IT.vectors.separate.2)
  plot2.title=plot2+ggtitle(label="asymmetric, global vaccination\nrate $\\nu_{\\rm tot}=0.004$ (per day)")+
    theme(plot.title=element_text(hjust=0.5,face="bold",size=12))
  nu.value.modif=0.001
  IT.vectors.separate.3=computing.IT.separate(nu.total=nu.value.modif,beta.p=beta.p.modif,
                                              beta.d=beta.d,delta.v=delta.v,
                                              delta=delta,epsilon=epsilon,phiBA=phiBA)
  plot3=plot.underlying(IT.vectors.separate.3)
  plot3.title=plot3+ggtitle(label="asymmetric, global vaccination\nrate $\\nu_{\\rm tot}=0.001$ (per day)")+
    theme(plot.title=element_text(hjust=0.5,face="bold",size=12))
  combined.all=(((reg.beta.title|higher.beta.title))/(plot1.title|plot2.title|plot3.title))+plot_layout(heights=c(2,1))+
    plot_annotation(tag_levels="A")&
    theme(plot.tag=element_text(size=22,face='bold'))
  return(combined.all)
}

figure2=combined.figure(phiBA=1)
figureS1=combined.figure(phiBA=0.5)
figureS2=combined.figure(phiBA=2)
tikz("Fig2-May9-May21-May26-June24.tex",width=15,height=10,standAlone=T)
print(figure2)
dev.off()
tikz("FigS1-May9-May21-May26-June24.tex",width=15,height=10,standAlone=T)
print(figureS1)
dev.off()
tikz("FigS2-May9-May21-May26-June24.tex",width=15,height=10,standAlone=T)
print(figureS2)
dev.off()

