#############################################Packages required########################################
library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)
library(pracma)
library(foreach)
library(doParallel)
library(tictoc)
library(matrixStats)
library(tidyr)
library(ggpubr)
library(ggpattern)
library(doRNG)

sirs.2patch.model.doses.evo<-function (t,x,parameters) {
  
  SPa=x[1]
  IPa=x[2]
  Ra=x[3]
  SSa=x[4]
  ISa=x[5]
  V1a=x[6]
  V2a=x[7]
  IVa=x[8]
  SS1a=x[9]
  SS2a=x[10]
  IS1a=x[11]
  IS2a=x[12]
  
  SPb=x[13]
  IPb=x[14]
  Rb=x[15]
  SSb=x[16]
  ISb=x[17]
  V1b=x[18]
  V2b=x[19]
  IVb=x[20]
  SS1b=x[21]
  SS2b=x[22]
  IS1b=x[23]
  IS2b=x[24]
  
  nNSa=x[25]
  nNSb=x[26]
  
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  
  R0base.ratio=parameters[["R0base.ratio"]]
  R0.list.a=parameters[["R0.list.a"]]
  beta.a=R0.list.a[t]*gamma
  R0.list.b=parameters[["R0.list.b"]]
  beta.b=R0.list.b[t]*gamma*R0base.ratio
  ###Reinfection
  alpha=parameters[["alpha"]]
  alphaV=parameters[["alphaV"]]
  alpha1=parameters[["alpha1"]]
  alpha2=parameters[["alpha2"]]
  epsilon=parameters[["epsilon"]]
  epsilonV1=parameters[["epsilonV1"]]
  epsilonV2=parameters[["epsilonV2"]]
  epsilon1=parameters[["epsilon1"]]
  epsilon2=parameters[["epsilon2"]]
  ###Social distancing
  R0red.list.a=parameters[["R0red.list.a"]]
  beta.a=beta.a*R0red.list.a[t]
  R0red.list.b=parameters[["R0red.list.b"]]
  beta.b=beta.b*R0red.list.b[t]
  ###Importation
  eta=parameters[["eta"]]
  eta=eta*min(R0red.list.a[t],R0red.list.b[t])
  ###Evolution
  evo.is=parameters[["evo.is"]]
  evo.is1=parameters[["evo.is1"]]
  evo.is2=parameters[["evo.is2"]]
  evo.cutoff=parameters[["evo.cutoff"]]
  evo.strainbeta=parameters[["evo.strainbeta"]]
  ###Vaccination
  rho1=parameters[["rho1"]]
  rho2=parameters[["rho2"]]
  d=parameters[["d"]]
  tvax=parameters[["tvax"]]
  Nb2Na=parameters[["Nb2Na"]]
  nu0tot1=parameters[["nu0tot1"]]
  nu0tot2=parameters[["nu0tot2"]]
  vaxfrac.a1=parameters[["vaxfrac.a1"]]
  vaxfrac.a2=parameters[["vaxfrac.a2"]]
  omega.a1=parameters[["omega.a1"]]
  omega.a2=parameters[["omega.a2"]]
  omega.b1=parameters[["omega.b1"]]
  omega.b2=parameters[["omega.b2"]]
  tswitch=parameters[["tswitch"]]
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Calculate vaccination parameters
  if(t<tswitch){
    nu.a=nu0tot1*vaxfrac.a1*0.5^(4*omega.a1)
    omega.a=omega.a1
    nu.b=nu0tot1*(1-vaxfrac.a1)/Nb2Na*0.5^(4*omega.b1)
    omega.b=omega.b1
  }else{
    nu.a=nu0tot2*vaxfrac.a2*0.5^(4*omega.a2)
    omega.a=omega.a2
    nu.b=nu0tot2*(1-vaxfrac.a2)/Nb2Na*0.5^(4*omega.b2)
    omega.b=omega.b2
  }
  
  #Calculate evolution
  randval=runif(1)
  bool=0
  if(randval<=0.1){
    bool=1
  }
  evopot.a=evo.is*ISa+evo.is1*IS1a+evo.is2*IS2a
  if(evopot.a>=evo.cutoff){
    dnNSadt=10*bool
  }else{
    dnNSadt=0
  }
  evopot.b=evo.is*ISb+evo.is1*IS1b+evo.is2*IS2b
  if(evopot.b>=evo.cutoff){
    dnNSbdt=10*bool
  }else{
    dnNSbdt=0
  }
  
  beta.a=beta.a*(evo.strainbeta)^(nNSa+nNSb)
  beta.b=beta.b*(evo.strainbeta)^(nNSa+nNSb)

  ###Equations
  ITa=IPa+alpha*ISa+alphaV*IVa+alpha1*IS1a+alpha2*IS2a
  ITb=IPb+alpha*ISb+alphaV*IVb+alpha1*IS1b+alpha2*IS2b
  #Region A
  dSPadt=mu-(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SPa-svax*nu.a*SPa
  dIPadt=(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*IPa
  dRadt=gamma*(IPa+ISa+IVa+IS1a+IS2a)-(delta+mu)*Ra
  dSSadt=delta*Ra-epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SSa-svax*nu.a*SSa
  dISadt=epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*ISa
  dV1adt=svax*nu.a*(SPa+d*SSa)-epsilonV1*beta.a*V1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(omega.a+rho1+mu)*V1a
  dV2adt=(1-d)*svax*nu.a*SSa+omega.a*V1a-epsilonV2*beta.a*V2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(rho2+mu)*V2a
  dIVadt=(epsilonV1*beta.a*V1a+epsilonV2*beta.a*V2a)*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IVa
  dSS1adt=rho1*V1a-epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS1a
  dSS2adt=rho2*V2a-epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS2a
  dIS1adt=epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS1a
  dIS2adt=epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS2a
  
  dxdt.a <- c(dSPadt,dIPadt,dRadt,dSSadt,dISadt,dV1adt,dV2adt,dIVadt,dSS1adt,dSS2adt,dIS1adt,dIS2adt)
  
  #Region B
  dSPbdt=mu-(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SPb-svax*nu.b*SPb
  dIPbdt=(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*IPb
  dRbdt=gamma*(IPb+ISb+IVb+IS1b+IS2b)-(delta+mu)*Rb
  dSSbdt=delta*Rb-epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SSb-svax*nu.b*SSb
  dISbdt=epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*ISb
  dV1bdt=svax*nu.b*(SPb+d*SSb)-epsilonV1*beta.b*V1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(omega.b+rho1+mu)*V1b
  dV2bdt=(1-d)*svax*nu.b*SSb+omega.b*V1b-epsilonV2*beta.b*V2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(rho2+mu)*V2b
  dIVbdt=(epsilonV1*beta.b*V1b+epsilonV2*beta.b*V2b)*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IVb
  dSS1bdt=rho1*V1b-epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS1b
  dSS2bdt=rho2*V2b-epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS2b
  dIS1bdt=epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS1b
  dIS2bdt=epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS2b
  
  dxdt.b <- c(dSPbdt,dIPbdt,dRbdt,dSSbdt,dISbdt,dV1bdt,dV2bdt,dIVbdt,dSS1bdt,dSS2bdt,dIS1bdt,dIS2bdt)
  
  dxdt <- c(dxdt.a,dxdt.b,dnNSadt,dnNSbdt)
  #disp(dxdt)
  list(dxdt)
}

sirs.2patch.model.dec.doses.evo<-function (t,x,parameters) {
  
  SPa=x[1]
  IPa=x[2]
  Ra=x[3]
  SSa=x[4]
  ISa=x[5]
  V1a=x[6]
  V2a=x[7]
  IVa=x[8]
  SS1a=x[9]
  SS2a=x[10]
  IS1a=x[11]
  IS2a=x[12]
  
  SPb=x[13]
  IPb=x[14]
  Rb=x[15]
  SSb=x[16]
  ISb=x[17]
  V1b=x[18]
  V2b=x[19]
  IVb=x[20]
  SS1b=x[21]
  SS2b=x[22]
  IS1b=x[23]
  IS2b=x[24]
  
  nNSa=x[25]
  nNSb=x[26]
  
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  
  R0base.ratio=parameters[["R0base.ratio"]]
  R0.list.a=parameters[["R0.list.a"]]
  beta.a=R0.list.a[t]*gamma
  R0.list.b=parameters[["R0.list.b"]]
  beta.b=R0.list.b[t]*gamma*R0base.ratio
  ###Reinfection
  alpha=parameters[["alpha"]]
  alphaV=parameters[["alphaV"]]
  alpha1=parameters[["alpha1"]]
  alpha2=parameters[["alpha2"]]
  epsilon=parameters[["epsilon"]]
  epsilonV1=parameters[["epsilonV1"]]
  epsilonV2=parameters[["epsilonV2"]]
  epsilon1=parameters[["epsilon1"]]
  epsilon2=parameters[["epsilon2"]]
  ###Social distancing
  R0red.list.a=parameters[["R0red.list.a"]]
  beta.a=beta.a*R0red.list.a[t]
  R0red.list.b=parameters[["R0red.list.b"]]
  beta.b=beta.b*R0red.list.b[t]
  ###Importation
  eta=parameters[["eta"]]
  eta=eta*min(R0red.list.a[t],R0red.list.b[t])
  ###Evolution
  evo.is=parameters[["evo.is"]]
  evo.is1=parameters[["evo.is1"]]
  evo.is2=parameters[["evo.is2"]]
  evo.cutoff=parameters[["evo.cutoff"]]
  evo.strainbeta=parameters[["evo.strainbeta"]]
  ###Vaccination
  rho1=parameters[["rho1"]]
  rho2=parameters[["rho2"]]
  d=parameters[["d"]]
  tvax=parameters[["tvax"]]
  Nb2Na=parameters[["Nb2Na"]]
  nu0tot1=parameters[["nu0tot1"]]
  nu0tot2=parameters[["nu0tot2"]]
  vaxfrac.a1=parameters[["vaxfrac.a1"]]
  vaxfrac.a2=parameters[["vaxfrac.a2"]]
  omega.a1=parameters[["omega.a1"]]
  omega.a2=parameters[["omega.a2"]]
  omega.b1=parameters[["omega.b1"]]
  omega.b2=parameters[["omega.b2"]]
  tswitch=parameters[["tswitch"]]
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Calculate vaccination parameters
  if(t<tswitch){
    nu.a=nu0tot1*vaxfrac.a1*0.5^(4*omega.a1)
    omega.a=omega.a1
    nu.b=nu0tot1*(1-vaxfrac.a1)/Nb2Na*0.5^(4*omega.b1)
    omega.b=omega.b1
  }else{
    nu.a=nu0tot2*vaxfrac.a2*0.5^(4*omega.a2)
    omega.a=omega.a2
    nu.b=nu0tot2*(1-vaxfrac.a2)/Nb2Na*0.5^(4*omega.b2)
    omega.b=omega.b2
  }
  
  #Calculate evolution
  randval=runif(1)
  bool=0
  if(randval<=0.1){
    bool=1
  }
  evopot.a=evo.is*ISa+evo.is1*IS1a+evo.is2*IS2a
  if(evopot.a>=evo.cutoff){
    dnNSadt=10*bool
  }else{
    dnNSadt=0
  }
  evopot.b=evo.is*ISb+evo.is1*IS1b+evo.is2*IS2b
  if(evopot.b>=evo.cutoff){
    dnNSbdt=10*bool
  }else{
    dnNSbdt=0
  }
  
  #beta.a=beta.a*(evo.strainbeta)^(nNSa)
  #beta.b=beta.b*(evo.strainbeta)^(nNSb)
  
  ###Equations
  ITa=IPa+alpha*ISa+alphaV*IVa+alpha1*IS1a+alpha2*IS2a
  ITb=IPb+alpha*ISb+alphaV*IVb+alpha1*IS1b+alpha2*IS2b
  #Region A
  dSPadt=mu-(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SPa-svax*nu.a*SPa
  dIPadt=(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*IPa
  dRadt=gamma*(IPa+ISa+IVa+IS1a+IS2a)-(delta+mu)*Ra
  dSSadt=delta*Ra-epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SSa-svax*nu.a*SSa
  dISadt=epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*ISa
  dV1adt=svax*nu.a*(SPa+d*SSa)-epsilonV1*beta.a*V1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(omega.a+rho1+mu)*V1a
  dV2adt=(1-d)*svax*nu.a*SSa+omega.a*V1a-epsilonV2*beta.a*V2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(rho2+mu)*V2a
  dIVadt=(epsilonV1*beta.a*V1a+epsilonV2*beta.a*V2a)*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IVa
  dSS1adt=rho1*V1a-epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS1a
  dSS2adt=rho2*V2a-epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS2a
  dIS1adt=epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS1a
  dIS2adt=epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS2a
  
  dxdt.a <- c(dSPadt,dIPadt,dRadt,dSSadt,dISadt,dV1adt,dV2adt,dIVadt,dSS1adt,dSS2adt,dIS1adt,dIS2adt)
  
  #Region B
  dSPbdt=mu-(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SPb-svax*nu.b*SPb
  dIPbdt=(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*IPb
  dRbdt=gamma*(IPb+ISb+IVb+IS1b+IS2b)-(delta+mu)*Rb
  dSSbdt=delta*Rb-epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SSb-svax*nu.b*SSb
  dISbdt=epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*ISb
  dV1bdt=svax*nu.b*(SPb+d*SSb)-epsilonV1*beta.b*V1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(omega.b+rho1+mu)*V1b
  dV2bdt=(1-d)*svax*nu.b*SSb+omega.b*V1b-epsilonV2*beta.b*V2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(rho2+mu)*V2b
  dIVbdt=(epsilonV1*beta.b*V1b+epsilonV2*beta.b*V2b)*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IVb
  dSS1bdt=rho1*V1b-epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS1b
  dSS2bdt=rho2*V2b-epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS2b
  dIS1bdt=epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS1b
  dIS2bdt=epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS2b
  
  dxdt.b <- c(dSPbdt,dIPbdt,dRbdt,dSSbdt,dISbdt,dV1bdt,dV2bdt,dIVbdt,dSS1bdt,dSS2bdt,dIS1bdt,dIS2bdt)
  
  dxdt <- c(dxdt.a,dxdt.b,dnNSadt,dnNSbdt)
  #disp(dxdt)
  list(dxdt)
}

totcases.nstrain.eff.vaxfrac=function(eta,R0base.ratio,vaxfrac.a1,vaxfrac.a2,evo.is,evo.is1,evo.is2){

  vaxfrac.a1=vaxfrac.a1
  vaxfrac.a2=vaxfrac.a2
  eta=eta
  R0base.ratio=R0base.ratio
  evo.is=evo.is
  evo.is1=evo.is1
  evo.is2=evo.is2
  
  #ODE ICs and parameters
  I0=1e-9
  xstart.a=c(SPa=1-I0,IPa=I0,Ra=0,SSa=0,ISa=0,V1a=0,V2a=0,IVa=0,SS1a=0,SS2a=0,IS1a=0,IS2a=0) 
  xstart.b=c(SPb=1-I0,IPb=I0,Rb=0,SSb=0,ISb=0,V1b=0,V2b=0,IVb=0,SS1b=0,SS2b=0,IS1b=0,IS2b=0)
  xstart=c(xstart.a,xstart.b,nNSa=0,nNSb=0)
  
  parameters=list(mu=mu,delta=delta,gamma=gamma,R0.list.a=R0.list.a,R0.list.b=R0.list.b,
                  alpha=alpha,alphaV=alphaV,alpha1=alpha1,alpha2=alpha2,epsilon=epsilon,
                  epsilonV1=epsilonV1,epsilonV2=epsilonV2,epsilon1=epsilon1,epsilon2=epsilon2,
                  R0base.ratio=R0base.ratio,R0red.list.a=R0red.list.a,R0red.list.b=R0red.list.b,eta=eta,
                  evo.is=evo.is,evo.is1=evo.is1,evo.is2=evo.is2,evo.cutoff=evo.cutoff,evo.strainbeta=evo.strainbeta,
                  rho1=rho1,rho2=rho2,d=d,tvax=tvax,Nb2Na=Nb2Na,nu0tot1=nu0tot1,nu0tot2=nu0tot2,
                  vaxfrac.a1=vaxfrac.a1,vaxfrac.a2=vaxfrac.a2,omega.a1=omega.a1,omega.a2=omega.a2,
                  omega.b1=omega.b1,omega.b2=omega.b2,tswitch=tswitch)
  
  out=as.data.frame(rk(func=sirs.2patch.model.doses.evo,y=xstart,times=times,
                            parms=parameters,hini=0.1,method="euler"))
  
  return(out)
}

totcases.nstrain.dec.eff.vaxfrac=function(eta,R0base.ratio,vaxfrac.a1,vaxfrac.a2,evo.is,evo.is1,evo.is2){

  vaxfrac.a1=vaxfrac.a1
  vaxfrac.a2=vaxfrac.a2
  eta=eta
  R0base.ratio=R0base.ratio
  evo.is=evo.is
  evo.is1=evo.is1
  evo.is2=evo.is2
  
  #ODE ICs and parameters
  I0=1e-9
  xstart.a=c(SPa=1-I0,IPa=I0,Ra=0,SSa=0,ISa=0,V1a=0,V2a=0,IVa=0,SS1a=0,SS2a=0,IS1a=0,IS2a=0) 
  xstart.b=c(SPb=1-I0,IPb=I0,Rb=0,SSb=0,ISb=0,V1b=0,V2b=0,IVb=0,SS1b=0,SS2b=0,IS1b=0,IS2b=0)
  xstart=c(xstart.a,xstart.b,nNSa=0,nNSb=0)
  
  parameters=list(mu=mu,delta=delta,gamma=gamma,R0.list.a=R0.list.a,R0.list.b=R0.list.b,
                  alpha=alpha,alphaV=alphaV,alpha1=alpha1,alpha2=alpha2,epsilon=epsilon,
                  epsilonV1=epsilonV1,epsilonV2=epsilonV2,epsilon1=epsilon1,epsilon2=epsilon2,
                  R0base.ratio=R0base.ratio,R0red.list.a=R0red.list.a,R0red.list.b=R0red.list.b,eta=eta,
                  evo.is=evo.is,evo.is1=evo.is1,evo.is2=evo.is2,evo.cutoff=evo.cutoff,evo.strainbeta=evo.strainbeta,
                  rho1=rho1,rho2=rho2,d=d,tvax=tvax,Nb2Na=Nb2Na,nu0tot1=nu0tot1,nu0tot2=nu0tot2,
                  vaxfrac.a1=vaxfrac.a1,vaxfrac.a2=vaxfrac.a2,omega.a1=omega.a1,omega.a2=omega.a2,
                  omega.b1=omega.b1,omega.b2=omega.b2,tswitch=tswitch)
  
  out=as.data.frame(rk(func=sirs.2patch.model.dec.doses.evo,y=xstart,times=times,
                       parms=parameters,hini=0.1,method="euler"))
  
  return(out)
}

############Set parameters##############
#Demographics
Nb2Na=1
mu=1/(50*52)
nyears=5

#Immunology
gamma=7/5
alpha=1
alphaV=alpha
alpha1=alpha
alpha2=alpha
epsilon=0.7
delta=1/(1*52)

#Transmission Rate Seasonality
times=seq(from=1,to=nyears*52,by=1)
R0.listOrig.nyc=read.table("NY.csv",header=T,sep=",")
R0.list.nyc=R0.listOrig.nyc$hku1*2.3/mean(R0.listOrig.nyc$hku1)

R0.list.nyc.shift <- circshift(R0.list.nyc,26)

R0.listOrig.del=read.table("Delhi.csv",header=T,sep=",")
R0.list.del=R0.listOrig.del$hku1*2.3/mean(R0.listOrig.del$hku1)

R0.listOrig.jak=read.table("Jakarta.csv",header=T,sep=",")
R0.list.jak=R0.listOrig.jak$hku1*2.3/mean(R0.listOrig.jak$hku1)

R0.list.a=rep(R0.list.nyc,length=length(times))
R0.list.b=rep(R0.list.nyc,length=length(times))

#Construct Social Distancing
tred1.a=7
tred2.a=45
tred3.a=80
redval1.a=0.40
redval2.a=0.60

R0red.list.a=rep(1,length(times))
R0red.list.a[(tred1.a+1):(tred2.a-1)]=rep(redval1.a,(tred2.a-tred1.a-1))
R0red.list.a[tred2.a:(tred3.a-1)]=rep(redval2.a,(tred3.a-tred2.a))

tred1.b=7
tred2.b=45
tred3.b=80
redval1.b=0.40
redval2.b=0.60

R0red.list.b=rep(1,length(times))
R0red.list.b[(tred1.b+1):(tred2.b-1)]=rep(redval1.b,(tred2.b-tred1.b-1))
R0red.list.b[tred2.b:(tred3.b-1)]=rep(redval2.b,(tred3.b-tred2.b))

#Clinical Considerations
primary.burden = 0.14
secondary.burden = 0.07
novax.burden = 0.14
twodose.burden = 0

#Evolution
evo.cutoff=0.01
evo.strainbeta=1

#Vaccinations
d=0.5
tvax=48
omega.a1=1/4
omega.a2=1/4
omega.b1=1/4
omega.b2=1/4
nu0tot1=0.02
nu0tot2=0.02
tswitch=48
epsilon2=0.7
epsilonV1=0.1
epsilonV2=0.05
rho2=1/(1*52)
d1d2=0.8
rho1=rho2/d1d2
epsilon1=epsilon2+(1-d1d2)*(1-epsilon2)
onedose.burden = twodose.burden + (novax.burden-twodose.burden) * (1-d1d2)

#Immigration
eta.base=0.01

#Loop specifications
nloop=100

##############Set parameters################
#####Chose whether natural infection or infection after waned vaccine is worse for evolution
##Worse natural infection
evo.is=0.8
evo.is1=0.2/d1d2
evo.is2=0.2

##Worse after vaccine
#evo.is=0.4
#evo.is1=0.8
#evo.is2=0.8*d1d2

#####Chose whether you are varying immigration or base R0
R0base.ratio=1.2
eta=eta.base*1

#####Chose fraction of vaccines retained by country A
vaxfrac.a1=0.5
vaxfrac.a2=0.5

##############Run loops################
##Coupled
#tic()
cores=detectCores()
cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
doParallel::registerDoParallel(cl)

set.seed(123)
out.combined <- foreach(k=1:nloop, .combine=cbind, .packages=c("deSolve","dplyr")) %dorng% {
  out <- totcases.nstrain.eff.vaxfrac(eta,R0base.ratio,vaxfrac.a1,vaxfrac.a2,evo.is,evo.is1,evo.is2)
  tot.a <- out$IPa + out$ISa + out$IVa + out$IS1a + out$IS2a
  tot.b <- out$IPb + out$ISb + out$IVb + out$IS1b + out$IS2b
  tot<-tot.a+tot.b
  evo.a <- evo.is*out$ISa+evo.is1*out$IS1a+evo.is2*out$IS2a
  evo.b <- evo.is*out$ISb+evo.is1*out$IS1b+evo.is2*out$IS2b
  out2<-bind_cols(out,tot.a,tot.b,tot,evo.a,evo.b)
  out2
}
stopCluster(cl)
#toc()

##Decoupled
#tic()
cores=detectCores()
cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
doParallel::registerDoParallel(cl)

set.seed(123)
out.combined.dec <- foreach(k=1:nloop, .combine=cbind, .packages=c("deSolve","dplyr")) %dorng% {
  out <- totcases.nstrain.dec.eff.vaxfrac(0,R0base.ratio,vaxfrac.a1,vaxfrac.a2,evo.is,evo.is1,evo.is2)
  tot.a <- out$IPa + out$ISa + out$IVa + out$IS1a + out$IS2a
  tot.b <- out$IPb + out$ISb + out$IVb + out$IS1b + out$IS2b
  tot<-tot.a+tot.b
  evo.a <- evo.is*out$ISa+evo.is1*out$IS1a+evo.is2*out$IS2a
  evo.b <- evo.is*out$ISb+evo.is1*out$IS1b+evo.is2*out$IS2b
  out2<-bind_cols(out,tot.a,tot.b,tot,evo.a,evo.b)
  out2
}
stopCluster(cl)
#toc()

out.mean <- matrix(rep(0,(length(times)*32)),ncol=32)
out.var <- matrix(rep(0,(length(times)*32)),ncol=32)
out.mean[,1] <- times
out.var[,1] <- times

out.dec.mean <- matrix(rep(0,(length(times)*32)),ncol=32)
out.dec.var <- matrix(rep(0,(length(times)*32)),ncol=32)
out.dec.mean[,1] <- times
out.dec.var[,1] <- times

for (i in 1:31){
  temp<- matrix(rep(0,(length(times)*nloop)),ncol=nloop)
  temp.dec<- matrix(rep(0,(length(times)*nloop)),ncol=nloop)
  for(j in 1:nloop){
      temp[,j]<-matrix(out.combined[,((i+1)+(j-1)*32)])
      temp.dec[,j]<-matrix(out.combined.dec[,((i+1)+(j-1)*32)])
  }
  out.mean[,(i+1)]<-rowMeans(temp)
  out.var[,(i+1)]<-rowVars(temp)
  out.dec.mean[,(i+1)]<-rowMeans(temp.dec)
  out.dec.var[,(i+1)]<-rowVars(temp.dec)
}
names<-c("times","SPa", "IPa", "Ra", "SSa", "ISa", "V1a", "V2a", "IVa", "SS1a", "SS2a", "IS1a","IS2a",
         "SPb", "IPb", "Rb", "SSb", "ISb", "V1b", "V2b", "IVb", "SS1b", "SS2b", "IS1b","IS2b", "nNSa",
         "nNSb", "Tot.a", "Tot.b", "Tot", "Evo.a","Evo.b")
  
out.mean <- as.data.frame(out.mean)
colnames(out.mean) <- names
out.var <- as.data.frame(out.var)
colnames(out.var) <- names

out.dec.mean <- as.data.frame(out.dec.mean)
colnames(out.dec.mean) <- names
out.dec.var <- as.data.frame(out.dec.var)
colnames(out.dec.var) <- names

#############Cases###########
###Coupled
#Total cases in A
totcases.a <- out.mean$Tot.a
sdcases.a <- sqrt(out.var$Tot.a)
#Total cases in B
totcases.b <- out.mean$Tot.b
sdcases.b <- sqrt(out.var$Tot.b)
#Total cases
totcases <- out.mean$Tot
sdcases <- sqrt(out.var$Tot)
###Decoupled
#Total cases in A
totcases.dec.a <- out.dec.mean$Tot.a
sdcases.dec.a <- sqrt(out.dec.var$Tot.a)
#Total cases in B
totcases.dec.b <- out.dec.mean$Tot.b
sdcases.dec.b <- sqrt(out.dec.var$Tot.b)
#Total cases
totcases.dec <- out.dec.mean$Tot
sdcases.dec <- sqrt(out.dec.var$Tot)
###########Evolutionary Pressure###########
##Coupled
#In A
evopress.a <- out.mean$Evo.a
sdevopress.a <- sqrt(out.var$Evo.a)
#In B
evopress.b <- out.mean$Evo.b
sdevopress.b <- sqrt(out.var$Evo.b)
##Decoupled
#In A
evopress.dec.a <- out.dec.mean$Evo.a
sdevopress.dec.a <- sqrt(out.dec.var$Evo.a)
#In B
evopress.dec.b <- out.dec.mean$Evo.b
sdevopress.dec.b <- sqrt(out.dec.var$Evo.b)

###########Number of Strains###########
##Coupled
#In A
nstrain.a <- out.mean$nNSa
sdstrain.a <- sqrt(out.var$nNSa)
#In B
nstrain.b <- out.mean$nNSb
sdstrain.b <- sqrt(out.var$nNSb)
#Total
nstrain <- out.mean$nNSa + out.mean$nNSb
##Decoupled
#In A
nstrain.dec.a <- out.dec.mean$nNSa
sdstrain.dec.a <- sqrt(out.dec.var$nNSa)
#In B
nstrain.dec.b <- out.dec.mean$nNSb
sdstrain.dec.b <- sqrt(out.dec.var$nNSb)
#Total
nstrain.dec <- out.dec.mean$nNSa + out.dec.mean$nNSb

######Make data frame for plotting######
###Coupled
#Cases
data.summary.cases <-cbind.data.frame(times=out.mean$times,totcases.a,low.a=totcases.a-sdcases.a, high.a=totcases.a+sdcases.a,
                                      totcases.b,low.b=totcases.b-sdcases.b, high.b=totcases.b+sdcases.b,
                                      totcases,low=totcases-sdcases, high=totcases+sdcases)
#Strains
data.summary.strains <-cbind.data.frame(times=out.mean$times,nstrain.a,low.a=nstrain.a-sdstrain.a, high.a=nstrain.a+sdstrain.a,
                                      nstrain.b,low.b=nstrain.b-sdstrain.b, high.b=nstrain.b+sdstrain.b,
                                      nstrain,low=nstrain-sdstrain, high=nstrain+sdstrain)
#Evolutionary Pressure
data.summary.evopress <-cbind.data.frame(times=out.mean$times,evopress.a,low.a=evopress.a-sdevopress.a, high.a=evopress.a+sdevopress.a,
                                        evopress.b,low.b=evopress.b-sdevopress.b, high.b=evopress.b+sdevopress.b)
###Decoupled
#Cases
data.summary.dec.cases <-cbind.data.frame(times=out.dec.mean$times,totcases.dec.a,low.a=totcases.dec.a-sdcases.dec.a, high.a=totcases.dec.a+sdcases.dec.a,
                                      totcases.dec.b,low.b=totcases.dec.b-sdcases.dec.b, high.b=totcases.dec.b+sdcases.dec.b,
                                      totcases.dec,low=totcases.dec-sdcases.dec, high=totcases.dec+sdcases.dec)
#Strains
data.summary.dec.strains <-cbind.data.frame(times=out.dec.mean$times,nstrain.dec.a,low.a=nstrain.dec.a-sdstrain.dec.a, high.a=nstrain.dec.a+sdstrain.dec.a,
                                        nstrain.dec.b,low.b=nstrain.dec.b-sdstrain.dec.b, high.b=nstrain.dec.b+sdstrain.dec.b,
                                        nstrain.dec,low=nstrain.dec-sdstrain.dec, high=nstrain.dec+sdstrain.dec)
#Evolutionary Pressure
data.summary.dec.evopress <-cbind.data.frame(times=out.dec.mean$times,evopress.dec.a,low.a=evopress.dec.a-sdevopress.dec.a, high.a=evopress.dec.a+sdevopress.dec.a,
                                         evopress.dec.b,low.b=evopress.dec.b-sdevopress.dec.b, high.b=evopress.dec.b+sdevopress.dec.b)


########Plotting##########
##Cases
#Coupled
cases.plot=ggplot(data=data.summary.cases)+geom_ribbon(aes(x=times,ymin=low.a,ymax=high.a), fill = "lightblue1")+
  geom_line(aes(x=times,y=totcases.a),linetype=1,color="blue3",alpha=0.8,size=1)+
  geom_ribbon(aes(x=times,ymin=low.b,ymax=high.b), fill = "rosybrown1")+
  geom_line(aes(x=times,y=totcases.b),linetype=1,color="red3",alpha=0.6,size=1)+
  ylim(-0.005,0.15) + xlab("Time (weeks)")+ylab("Cases") + theme_bw()+
  labs(title="Coupled")+theme(plot.title = element_text(size=14, face="bold",hjust = 0.5, vjust=2))+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 12),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 12 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 12,face="bold") 
  )
#Decoupled
cases.dec.plot=ggplot(data=data.summary.dec.cases)+geom_ribbon(aes(x=times,ymin=low.a,ymax=high.a), fill = "lightblue1")+
  geom_line(aes(x=times,y=totcases.dec.a),linetype=1,color="blue3", alpha=0.8, size=1)+
  geom_ribbon(aes(x=times,ymin=low.b,ymax=high.b), fill = "rosybrown1")+
  geom_line(aes(x=times,y=totcases.dec.b),linetype=1,color="red3", alpha=0.6,size=1)+
  ylim(-0.005,0.15) + xlab("Times (weeks)")+ylab("Cases") + theme_bw()+
  labs(title="Decoupled")+theme(plot.title = element_text(size=14, face="bold",hjust = 0.5, vjust=2))+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 12 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 12 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 12,face="bold") 
  )

##Total cases
#Coupled
sumcases.a <- gamma*sum(totcases.a[tvax:length(times)])
sumcases.b <- gamma*sum(totcases.b[tvax:length(times)])
sumcases <- gamma*sum(totcases[tvax:length(times)])
frame.sumcases <- data.frame(placevec=c("HAR", "LAR", "Total"),casevec=c(sumcases.a, sumcases.b,sumcases))
temp<-melt(frame.sumcases,id.vars ="placevec")
temp2<-cbind.data.frame(temp,Method=c("Coupled","Coupled","Coupled"))
#Decoupled
sumcases.dec.a <- gamma*sum(totcases.dec.a[tvax:length(times)])
sumcases.dec.b <- gamma*sum(totcases.dec.b[tvax:length(times)])
sumcases.dec <- gamma*sum(totcases.dec[tvax:length(times)])
frame.sumcases.dec <- data.frame(placevec=c("HAR", "LAR", "Total"),casevec=c(sumcases.dec.a, sumcases.dec.b,sumcases.dec))
temp.dec<-melt(frame.sumcases.dec,id.vars ="placevec")
temp2.dec<-cbind.data.frame(temp.dec,Method=c("Decoupled","Decoupled","Decoupled"))

##Plot Total Cases Bar Plot
frame.sumcases.combined<-rbind.data.frame(temp2,temp2.dec)
sumcases.plot.combined=ggplot(data=frame.sumcases.combined, aes(x=placevec,y=value,fill=placevec,pattern=Method,alpha=placevec)) +
  geom_col_pattern(position = position_dodge(preserve = "single"),
                   color = "black", 
                   pattern_fill = "black",
                   pattern_angle = 45,
                   pattern_density = 0.1,
                   pattern_spacing = 0.025,
                   pattern_key_scale_factor = 0.6) + 
  xlab("Place") + ylab("") + ggtitle("Total cases") + theme_bw() + ylim(0,5) +
  scale_fill_manual(values = c("blue3", "red3","grey70"))+ 
  scale_alpha_manual(values=c(0.8,0.6,1))+
  scale_pattern_manual(values = c("Coupled" = "none","Decoupled" = "stripe")) +
  theme(plot.title = element_text(hjust = 0.5),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        legend.position = "none",
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text(size = 12 , face="bold"),
        title = element_text(size = 12 , face="bold"),
        axis.title = element_text(size = 12, face="bold")
  )

##Evolutionary pressure
#Coupled
evopress.plot=ggplot(data=data.summary.evopress)+geom_ribbon(aes(x=times,ymin=low.a,ymax=high.a), fill = "lightblue1")+
  geom_line(aes(x=times,y=evopress.a),linetype=1,color="blue3",alpha=0.8,size=1)+
  geom_ribbon(aes(x=times,ymin=low.b,ymax=high.b), fill = "rosybrown1")+
  geom_line(aes(x=times,y=evopress.b),linetype=1,color="red3",alpha=0.6,size=1)+
  geom_hline(yintercept=evo.cutoff,linetype=2)+
  ylim(-0.005,0.04) + xlab("Times (weeks)")+ylab(expression(atop("Potential net viral", paste("adaptation rate")))) + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 12 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 12 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 12,face="bold") 
  )
#Decoupled
evopress.dec.plot=ggplot(data=data.summary.dec.evopress)+geom_ribbon(aes(x=times,ymin=low.a,ymax=high.a), fill = "lightblue1")+
  geom_line(aes(x=times,y=evopress.dec.a),linetype=1,color="blue3",alpha=0.8,size=1)+
  geom_ribbon(aes(x=times,ymin=low.b,ymax=high.b), fill = "rosybrown1")+
  geom_line(aes(x=times,y=evopress.dec.b),linetype=1,color="red3",alpha=0.6,size=1)+
  geom_hline(yintercept=evo.cutoff,linetype=2)+
  ylim(-0.005,0.04) + xlab("Times (weeks)")+ylab(expression(atop("Potential net viral", paste("adaptation rate")))) + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 12 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 12 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 12,face="bold") 
  )

##Total strains
#Coupled
sumstrains.a <- nstrain.a[length(times)]
sumstrains.b <- nstrain.b[length(times)]
frame.sumstrains <- data.frame(placevec=c("HAR", "LAR"),strainvec=c(sumstrains.a, sumstrains.b))
temp<-melt(frame.sumstrains,id.vars ="placevec")
temp2<-cbind.data.frame(temp,Method=c("Coupled","Coupled"))
#Decoupled
sumstrains.dec.a <- nstrain.dec.a[length(times)]
sumstrains.dec.b <- nstrain.dec.b[length(times)]
frame.sumstrains.dec <- data.frame(placevec=c("HAR", "LAR"),strainvec=c(sumstrains.dec.a, sumstrains.dec.b))
temp.dec<-melt(frame.sumstrains.dec,id.vars ="placevec")
temp2.dec<-cbind.data.frame(temp.dec,Method=c("Decoupled","Decoupled"))

##Plot Total Strains Bar Plot
frame.sumstrains.combined<-rbind.data.frame(temp2,temp2.dec)
sumstrains.plot.combined=ggplot(data=frame.sumstrains.combined, aes(x=placevec,y=value,fill=placevec,pattern=Method,alpha=placevec)) +
  geom_col_pattern(position = position_dodge(preserve = "single"),
                   color = "black", 
                   pattern_fill = "black",
                   pattern_angle = 45,
                   pattern_density = 0.1,
                   pattern_spacing = 0.025,
                   pattern_key_scale_factor = 0.6) + 
  xlab("Place") + ylab("") + ggtitle("Strains") + theme_bw() + ylim(0,50) +
  scale_fill_manual(values = c("blue3", "red3"))+ 
  scale_alpha_manual(values=c(0.8,0.6))+
  scale_pattern_manual(values = c("Coupled" = "none", "Decoupled" = "stripe")) +
  theme(plot.title = element_text(hjust = 0.5),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        legend.position = "none",
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text(size = 12 , face="bold"),
        title = element_text(size = 12 , face="bold"),
        axis.title = element_text(size = 12, face="bold")
  )

#Combine plots
couple.plot<-plot_grid(cases.plot,evopress.plot,labels=c("A","B"), ncol=1, align="v", axis="l")
decouple.plot<-plot_grid(cases.dec.plot,evopress.dec.plot,labels=c("C","D"), ncol=1, align="v", axis="l")
totals.plot<-plot_grid(sumcases.plot.combined,sumstrains.plot.combined,labels=c("E","F"), ncol=1, align="v", axis="l")
all.plot<-ggarrange(couple.plot,decouple.plot,totals.plot,ncol=3)
all.plot

#pdf("CasesStrains_NoPTI_BNI_eta1_R01p2_vfrac0p5_May262021.pdf",width=12,height=6)
#print(all.plot)
#dev.off()

