#############################################Packages required########################################
library(deSolve)
library(ggplot2)
library(reshape2)
library(tidyr)
library(ggpubr)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)
library(pracma)
library(foreach)
library(doParallel)
library(tictoc)
library(doRNG)

sirs.2patch.model.doses.evo<-function (t,x,parameters) {
  
  SPa=x[1]
  IPa=x[2]
  Ra=x[3]
  SSa=x[4]
  ISa=x[5]
  V1a=x[6]
  V2a=x[7]
  IVa=x[8]
  SS1a=x[9]
  SS2a=x[10]
  IS1a=x[11]
  IS2a=x[12]
  
  SPb=x[13]
  IPb=x[14]
  Rb=x[15]
  SSb=x[16]
  ISb=x[17]
  V1b=x[18]
  V2b=x[19]
  IVb=x[20]
  SS1b=x[21]
  SS2b=x[22]
  IS1b=x[23]
  IS2b=x[24]
  
  nNSa=x[25]
  nNSb=x[26]
  
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  
  R0base.ratio=parameters[["R0base.ratio"]]
  R0.list.a=parameters[["R0.list.a"]]
  beta.a=R0.list.a[t]*gamma
  R0.list.b=parameters[["R0.list.b"]]
  beta.b=R0.list.b[t]*gamma*R0base.ratio
  ###Reinfection
  alpha=parameters[["alpha"]]
  alphaV=parameters[["alphaV"]]
  alpha1=parameters[["alpha1"]]
  alpha2=parameters[["alpha2"]]
  epsilon=parameters[["epsilon"]]
  epsilonV1=parameters[["epsilonV1"]]
  epsilonV2=parameters[["epsilonV2"]]
  epsilon1=parameters[["epsilon1"]]
  epsilon2=parameters[["epsilon2"]]
  ###Social distancing
  R0red.list.a=parameters[["R0red.list.a"]]
  beta.a=beta.a*R0red.list.a[t]
  R0red.list.b=parameters[["R0red.list.b"]]
  beta.b=beta.b*R0red.list.b[t]
  ###Importation
  eta=parameters[["eta"]]
  eta=eta*min(R0red.list.a[t],R0red.list.b[t])
  ###Evolution
  evo.is=parameters[["evo.is"]]
  evo.is1=parameters[["evo.is1"]]
  evo.is2=parameters[["evo.is2"]]
  evo.cutoff=parameters[["evo.cutoff"]]
  evo.strainbeta=parameters[["evo.strainbeta"]]
  ###Vaccination
  rho1=parameters[["rho1"]]
  rho2=parameters[["rho2"]]
  d=parameters[["d"]]
  tvax=parameters[["tvax"]]
  Nb2Na=parameters[["Nb2Na"]]
  nu0tot1=parameters[["nu0tot1"]]
  nu0tot2=parameters[["nu0tot2"]]
  vaxfrac.a1=parameters[["vaxfrac.a1"]]
  vaxfrac.a2=parameters[["vaxfrac.a2"]]
  omega.a1=parameters[["omega.a1"]]
  omega.a2=parameters[["omega.a2"]]
  omega.b1=parameters[["omega.b1"]]
  omega.b2=parameters[["omega.b2"]]
  tswitch=parameters[["tswitch"]]
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Calculate vaccination parameters
  if(t<tswitch){
    nu.a=nu0tot1*vaxfrac.a1*0.5^(4*omega.a1)
    omega.a=omega.a1
    nu.b=nu0tot1*(1-vaxfrac.a1)/Nb2Na*0.5^(4*omega.b1)
    omega.b=omega.b1
  }else{
    nu.a=nu0tot2*vaxfrac.a2*0.5^(4*omega.a2)
    omega.a=omega.a2
    nu.b=nu0tot2*(1-vaxfrac.a2)/Nb2Na*0.5^(4*omega.b2)
    omega.b=omega.b2
  }
  
  #Calculate evolution
  randval=runif(1)
  bool=0
  if(randval<=0.1){
    bool=1
  }
  evopot.a=evo.is*ISa+evo.is1*IS1a+evo.is2*IS2a
  if(evopot.a>=evo.cutoff){
    dnNSadt=10*bool
  }else{
    dnNSadt=0
  }
  evopot.b=evo.is*ISb+evo.is1*IS1b+evo.is2*IS2b
  if(evopot.b>=evo.cutoff){
    dnNSbdt=10*bool
  }else{
    dnNSbdt=0
  }
  
  beta.a=beta.a*(evo.strainbeta)^(nNSa+nNSb)
  beta.b=beta.b*(evo.strainbeta)^(nNSa+nNSb)
  
  ###Equations
  ITa=IPa+alpha*ISa+alphaV*IVa+alpha1*IS1a+alpha2*IS2a
  ITb=IPb+alpha*ISb+alphaV*IVb+alpha1*IS1b+alpha2*IS2b
  #Region A
  dSPadt=mu-(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SPa-svax*nu.a*SPa
  dIPadt=(beta.a*SPa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*IPa
  dRadt=gamma*(IPa+ISa+IVa+IS1a+IS2a)-(delta+mu)*Ra
  dSSadt=delta*Ra-epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-mu*SSa-svax*nu.a*SSa
  dISadt=epsilon*(beta.a*SSa*((1-eta)*ITa+eta*ITb*Nb2Na))-(gamma+mu)*ISa
  dV1adt=svax*nu.a*(SPa+d*SSa)-epsilonV1*beta.a*V1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(omega.a+rho1+mu)*V1a
  dV2adt=(1-d)*svax*nu.a*SSa+omega.a*V1a-epsilonV2*beta.a*V2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(rho2+mu)*V2a
  dIVadt=(epsilonV1*beta.a*V1a+epsilonV2*beta.a*V2a)*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IVa
  dSS1adt=rho1*V1a-epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS1a
  dSS2adt=rho2*V2a-epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-mu*SS2a
  dIS1adt=epsilon1*beta.a*SS1a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS1a
  dIS2adt=epsilon2*beta.a*SS2a*((1-eta)*ITa+eta*ITb*Nb2Na)-(gamma+mu)*IS2a
  
  dxdt.a <- c(dSPadt,dIPadt,dRadt,dSSadt,dISadt,dV1adt,dV2adt,dIVadt,dSS1adt,dSS2adt,dIS1adt,dIS2adt)
  
  #Region B
  dSPbdt=mu-(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SPb-svax*nu.b*SPb
  dIPbdt=(beta.b*SPb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*IPb
  dRbdt=gamma*(IPb+ISb+IVb+IS1b+IS2b)-(delta+mu)*Rb
  dSSbdt=delta*Rb-epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-mu*SSb-svax*nu.b*SSb
  dISbdt=epsilon*(beta.b*SSb*((1-eta)*ITb+eta*ITa/Nb2Na))-(gamma+mu)*ISb
  dV1bdt=svax*nu.b*(SPb+d*SSb)-epsilonV1*beta.b*V1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(omega.b+rho1+mu)*V1b
  dV2bdt=(1-d)*svax*nu.b*SSb+omega.b*V1b-epsilonV2*beta.b*V2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(rho2+mu)*V2b
  dIVbdt=(epsilonV1*beta.b*V1b+epsilonV2*beta.b*V2b)*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IVb
  dSS1bdt=rho1*V1b-epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS1b
  dSS2bdt=rho2*V2b-epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-mu*SS2b
  dIS1bdt=epsilon1*beta.b*SS1b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS1b
  dIS2bdt=epsilon2*beta.b*SS2b*((1-eta)*ITb+eta*ITa/Nb2Na)-(gamma+mu)*IS2b
  
  dxdt.b <- c(dSPbdt,dIPbdt,dRbdt,dSSbdt,dISbdt,dV1bdt,dV2bdt,dIVbdt,dSS1bdt,dSS2bdt,dIS1bdt,dIS2bdt)
  
  dxdt <- c(dxdt.a,dxdt.b,dnNSadt,dnNSbdt)
  #disp(dxdt)
  list(dxdt)
}

totcases.nstrain.vaxfrac=function(eta,R0base.ratio,vaxfrac.a1,vaxfrac.a2,
                                      evo.is,evo.is1,evo.is2){
  
  vaxfrac.a1=vaxfrac.a1
  vaxfrac.a2=vaxfrac.a2
  eta=eta
  R0base.ratio=R0base.ratio
  evo.is=evo.is
  evo.is1=evo.is1
  evo.is2=evo.is2
  
  #ODE ICs and parameters
  I0=1e-9
  xstart.a=c(SPa=1-I0,IPa=I0,Ra=0,SSa=0,ISa=0,V1a=0,V2a=0,IVa=0,SS1a=0,SS2a=0,IS1a=0,IS2a=0) 
  xstart.b=c(SPb=1-I0,IPb=I0,Rb=0,SSb=0,ISb=0,V1b=0,V2b=0,IVb=0,SS1b=0,SS2b=0,IS1b=0,IS2b=0)
  xstart=c(xstart.a,xstart.b,nNSa=0,nNSb=0)
  
  parameters=list(mu=mu,delta=delta,gamma=gamma,R0.list.a=R0.list.a,R0.list.b=R0.list.b,
                  alpha=alpha,alphaV=alphaV,alpha1=alpha1,alpha2=alpha2,epsilon=epsilon,
                  epsilonV1=epsilonV1,epsilonV2=epsilonV2,epsilon1=epsilon1,epsilon2=epsilon2,
                  R0base.ratio=R0base.ratio,R0red.list.a=R0red.list.a,R0red.list.b=R0red.list.b,eta=eta,
                  evo.is=evo.is,evo.is1=evo.is1,evo.is2=evo.is2,evo.cutoff=evo.cutoff,evo.strainbeta=evo.strainbeta,
                  rho1=rho1,rho2=rho2,d=d,tvax=tvax,Nb2Na=Nb2Na,nu0tot1=nu0tot1,nu0tot2=nu0tot2,
                  vaxfrac.a1=vaxfrac.a1,vaxfrac.a2=vaxfrac.a2,omega.a1=omega.a1,omega.a2=omega.a2,
                  omega.b1=omega.b1,omega.b2=omega.b2,tswitch=tswitch)
  
  out=as.data.frame(rk(func=sirs.2patch.model.doses.evo,y=xstart,times=times,
                       parms=parameters,hini=0.1,method="euler"))
  
  value.tot.a=gamma*sum(out$IPa[tvax:length(times)]+out$ISa[tvax:length(times)]+
                          out$IVa[tvax:length(times)]+out$IS1a[tvax:length(times)]+out$IS2a[tvax:length(times)])
  
  value.sev.a=gamma*sum(primary.burden*out$IPa[tvax:length(times)]+secondary.burden*out$ISa[tvax:length(times)]+
                          novax.burden*out$IVa[tvax:length(times)]+onedose.burden*out$IS1a[tvax:length(times)]+
                          twodose.burden*out$IS2a[tvax:length(times)])
  
  value.tot.b=gamma*sum(out$IPb[tvax:length(times)]+out$ISb[tvax:length(times)]+
                          out$IVb[tvax:length(times)]+out$IS1b[tvax:length(times)]+out$IS2b[tvax:length(times)])
  
  value.sev.b=gamma*sum(primary.burden*out$IPb[tvax:length(times)]+secondary.burden*out$ISb[tvax:length(times)]+
                          novax.burden*out$IVb[tvax:length(times)]+onedose.burden*out$IS1b[tvax:length(times)]+
                          twodose.burden*out$IS2b[tvax:length(times)])
  
  
  value.tot = value.tot.a + value.tot.b
  
  value.sev=value.sev.a+value.sev.b
  
  value.nstrain=out$nNSa[length(times)]+out$nNSb[length(times)]
  
  return(list(tot=value.tot,sev=value.sev,tota=value.tot.a,seva=value.sev.a,totb=value.tot.b,sevb=value.sev.b,strain=value.nstrain))
}

###Set parameters
#Demographics
Nb2Na=0.5
mu=1/(50*52)
nyears=5

#Immunology
gamma=7/5
alpha=1
alphaV=alpha
alpha1=alpha
alpha2=alpha
epsilon=0.7
delta=1/(1*52)

#Transmission Rate Seasonality
times=seq(from=1,to=nyears*52,by=1)
R0.listOrig.nyc=read.table("NY.csv",header=T,sep=",")
R0.list.nyc=R0.listOrig.nyc$hku1*2.3/mean(R0.listOrig.nyc$hku1)

R0.listOrig.del=read.table("Delhi.csv",header=T,sep=",")
R0.list.del=R0.listOrig.del$hku1*2.3/mean(R0.listOrig.del$hku1)

R0.listOrig.jak=read.table("Jakarta.csv",header=T,sep=",")
R0.list.jak=R0.listOrig.jak$hku1*2.3/mean(R0.listOrig.jak$hku1)

R0.list.a=rep(R0.list.nyc,length=length(times))
R0.list.b=rep(R0.list.nyc,length=length(times))

#Construct Social Distancing
tred1.a=7
tred2.a=45
tred3.a=80
redval1.a=0.40
redval2.a=0.60

R0red.list.a=rep(1,length(times))
R0red.list.a[(tred1.a+1):(tred2.a-1)]=rep(redval1.a,(tred2.a-tred1.a-1))
R0red.list.a[tred2.a:(tred3.a-1)]=rep(redval2.a,(tred3.a-tred2.a))

tred1.b=7
tred2.b=45
tred3.b=80
redval1.b=0.40
redval2.b=0.60

R0red.list.b=rep(1,length(times))
R0red.list.b[(tred1.b+1):(tred2.b-1)]=rep(redval1.b,(tred2.b-tred1.b-1))
R0red.list.b[tred2.b:(tred3.b-1)]=rep(redval2.b,(tred3.b-tred2.b))

#Clinical Considerations
primary.burden = 0.14
secondary.burden = 0.07
novax.burden = 0.14
twodose.burden = 0

#Evolution
evo.cutoff=0.01
evo.strainbeta=1.01

#Vaccinations
d=0.5
tvax=48
omega.a1=1/4
omega.a2=1/4
omega.b1=1/4
omega.b2=1/4
nu0tot1=0.02
nu0tot2=0.02
tswitch=48
epsilon2=0.7
epsilonV1=0.1
epsilonV2=0.05
rho2=1/(1*52)
d1d2=0.8
rho1=rho2/d1d2
epsilon1=epsilon2+(1-d1d2)*(1-epsilon2)
onedose.burden = twodose.burden + (novax.burden-twodose.burden) * (1-d1d2)

#Immigration
eta.base=0.01

#Loop Specifications
nticks=25
nloop=100

########################SCENARIOS############################
########Natural infections worse for evolution###############
###Vary immigration
evo.is=0.8
evo.is1=0.2/d1d2
evo.is2=0.2

R0base.ratio=1

vaxfrac.vector=seq(from=0,to=1,length.out=nticks)
eta.vector=seq(from=0,to=5,length.out=nticks) * eta.base

output.totcases.bni.imm <- matrix(0,nticks,nticks)
output.sevcases.bni.imm <- matrix(0,nticks,nticks)
output.totcases.a.bni.imm <- matrix(0,nticks,nticks)
output.sevcases.a.bni.imm <- matrix(0,nticks,nticks)
output.totcases.b.bni.imm <- matrix(0,nticks,nticks)
output.sevcases.b.bni.imm <- matrix(0,nticks,nticks)
output.nstrain.bni.imm <- matrix(0,nticks,nticks)

for(i in 1:nticks){
  for(j in 1:nticks){
    
    #tic()
    cores=detectCores()
    cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
    doParallel::registerDoParallel(cl)
    
    set.seed(123)
    randvec <- foreach(k=1:nloop, .combine=rbind, .packages=c("deSolve")) %dorng% {
      out <- totcases.nstrain.vaxfrac(eta.vector[i],R0base.ratio,vaxfrac.vector[j],
                                          vaxfrac.vector[j],evo.is,evo.is1,evo.is2)
      tempvec<-matrix(0,1,7)
      tempvec[1,1]<-out$tot
      tempvec[1,2]<-out$sev
      tempvec[1,3]<-out$tota
      tempvec[1,4]<-out$seva
      tempvec[1,5]<-out$totb
      tempvec[1,6]<-out$sevb
      tempvec[1,7]<-out$strain
      
      tempvec
    }
    stopCluster(cl)
    #toc()
    
    output.totcases.bni.imm[i,j] <- mean(randvec[,1])
    output.sevcases.bni.imm[i,j] <- mean(randvec[,2])
    output.totcases.a.bni.imm[i,j] <- mean(randvec[,3])
    output.sevcases.a.bni.imm[i,j] <- mean(randvec[,4])
    output.totcases.b.bni.imm[i,j] <- mean(randvec[,5])
    output.sevcases.b.bni.imm[i,j] <- mean(randvec[,6])
    output.nstrain.bni.imm[i,j] <- mean(randvec[,7])
  }
}
store.output.totcases.bni.imm <-output.totcases.bni.imm
store.output.sevcases.bni.imm <-output.sevcases.bni.imm
store.output.totcases.a.bni.imm <-output.totcases.a.bni.imm
store.output.sevcases.a.bni.imm <-output.sevcases.a.bni.imm
store.output.totcases.b.bni.imm <-output.totcases.b.bni.imm
store.output.sevcases.b.bni.imm <-output.sevcases.b.bni.imm
store.output.nstrain.bni.imm<-output.nstrain.bni.imm

###Vary R0
evo.is=0.8
evo.is1=0.2/d1d2
evo.is2=0.2

eta=eta.base*1

vaxfrac.vector=seq(from=0,to=1,length.out=nticks)
R0base.vector=seq(from=0.5,to=2,length.out=nticks)

output.totcases.bni.R0 <- matrix(0,nticks,nticks)
output.sevcases.bni.R0 <- matrix(0,nticks,nticks)
output.totcases.a.bni.R0 <- matrix(0,nticks,nticks)
output.sevcases.a.bni.R0 <- matrix(0,nticks,nticks)
output.totcases.b.bni.R0 <- matrix(0,nticks,nticks)
output.sevcases.b.bni.R0 <- matrix(0,nticks,nticks)
output.nstrain.bni.R0 <- matrix(0,nticks,nticks)

for(i in 1:nticks){
  for(j in 1:nticks){
    
    #tic()
    cores=detectCores()
    cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
    doParallel::registerDoParallel(cl)
    
    set.seed(123)
    randvec <- foreach(k=1:nloop, .combine=rbind, .packages=c("deSolve")) %dorng% {
      out <- totcases.nstrain.vaxfrac(eta,R0base.vector[i],vaxfrac.vector[j],
                                          vaxfrac.vector[j],evo.is,evo.is1,evo.is2)
      tempvec<-matrix(0,1,7)
      tempvec[1,1]<-out$tot
      tempvec[1,2]<-out$sev
      tempvec[1,3]<-out$tota
      tempvec[1,4]<-out$seva
      tempvec[1,5]<-out$totb
      tempvec[1,6]<-out$sevb
      tempvec[1,7]<-out$strain
      
      tempvec
    }
    stopCluster(cl)
    #toc()
    
    output.totcases.bni.R0[i,j] <- mean(randvec[,1])
    output.sevcases.bni.R0[i,j] <- mean(randvec[,2])
    output.totcases.a.bni.R0[i,j] <- mean(randvec[,3])
    output.sevcases.a.bni.R0[i,j] <- mean(randvec[,4])
    output.totcases.b.bni.R0[i,j] <- mean(randvec[,5])
    output.sevcases.b.bni.R0[i,j] <- mean(randvec[,6])
    output.nstrain.bni.R0[i,j] <- mean(randvec[,7])
  }
}
store.output.totcases.bni.R0 <-output.totcases.bni.R0
store.output.sevcases.bni.R0 <-output.sevcases.bni.R0
store.output.totcases.a.bni.R0 <-output.totcases.a.bni.R0
store.output.sevcases.a.bni.R0 <-output.sevcases.a.bni.R0
store.output.totcases.b.bni.R0 <-output.totcases.b.bni.R0
store.output.sevcases.b.bni.R0 <-output.sevcases.b.bni.R0
store.output.nstrain.bni.R0<-output.nstrain.bni.R0

########Post vaccine infections worse for evolution###############
###Vary immigration
evo.is=0.4
evo.is1=0.8
evo.is2=0.8*d1d2

R0base.ratio=1

vaxfrac.vector=seq(from=0,to=1,length.out=nticks)
eta.vector=seq(from=0,to=5,length.out=nticks) * eta.base

output.totcases.bv.imm <- matrix(0,nticks,nticks)
output.sevcases.bv.imm <- matrix(0,nticks,nticks)
output.totcases.a.bv.imm <- matrix(0,nticks,nticks)
output.sevcases.a.bv.imm <- matrix(0,nticks,nticks)
output.totcases.b.bv.imm <- matrix(0,nticks,nticks)
output.sevcases.b.bv.imm <- matrix(0,nticks,nticks)
output.nstrain.bv.imm <- matrix(0,nticks,nticks)

for(i in 1:nticks){
  for(j in 1:nticks){
    
    #tic()
    cores=detectCores()
    cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
    doParallel::registerDoParallel(cl)
    
    set.seed(123)
    randvec <- foreach(k=1:nloop, .combine=rbind, .packages=c("deSolve")) %dorng% {
      out <- totcases.nstrain.vaxfrac(eta.vector[i],R0base.ratio,vaxfrac.vector[j],
                                          vaxfrac.vector[j],evo.is,evo.is1,evo.is2)
      tempvec<-matrix(0,1,7)
      tempvec[1,1]<-out$tot
      tempvec[1,2]<-out$sev
      tempvec[1,3]<-out$tota
      tempvec[1,4]<-out$seva
      tempvec[1,5]<-out$totb
      tempvec[1,6]<-out$sevb
      tempvec[1,7]<-out$strain
      
      tempvec
    }
    stopCluster(cl)
    #toc()
    
    output.totcases.bv.imm[i,j] <- mean(randvec[,1])
    output.sevcases.bv.imm[i,j] <- mean(randvec[,2])
    output.totcases.a.bv.imm[i,j] <- mean(randvec[,3])
    output.sevcases.a.bv.imm[i,j] <- mean(randvec[,4])
    output.totcases.b.bv.imm[i,j] <- mean(randvec[,5])
    output.sevcases.b.bv.imm[i,j] <- mean(randvec[,6])
    output.nstrain.bv.imm[i,j] <- mean(randvec[,7])
  }
}
store.output.totcases.bv.imm <-output.totcases.bv.imm
store.output.sevcases.bv.imm <-output.sevcases.bv.imm
store.output.totcases.a.bv.imm <-output.totcases.a.bv.imm
store.output.sevcases.a.bv.imm <-output.sevcases.a.bv.imm
store.output.totcases.b.bv.imm <-output.totcases.b.bv.imm
store.output.sevcases.b.bv.imm <-output.sevcases.b.bv.imm
store.output.nstrain.bv.imm<-output.nstrain.bv.imm

###Vary R0
evo.is=0.4
evo.is1=0.8
evo.is2=0.8*d1d2

eta=eta.base*1

vaxfrac.vector=seq(from=0,to=1,length.out=nticks)
R0base.vector=seq(from=0.5,to=2,length.out=nticks)

output.totcases.bv.R0 <- matrix(0,nticks,nticks)
output.sevcases.bv.R0 <- matrix(0,nticks,nticks)
output.totcases.a.bv.R0 <- matrix(0,nticks,nticks)
output.sevcases.a.bv.R0 <- matrix(0,nticks,nticks)
output.totcases.b.bv.R0 <- matrix(0,nticks,nticks)
output.sevcases.b.bv.R0 <- matrix(0,nticks,nticks)
output.nstrain.bv.R0 <- matrix(0,nticks,nticks)

for(i in 1:nticks){
  for(j in 1:nticks){
    
    #tic()
    cores=detectCores()
    cl <- parallel::makeCluster((cores-1), setup_strategy = "sequential")
    doParallel::registerDoParallel(cl)
    
    set.seed(123)
    randvec <- foreach(k=1:nloop, .combine=rbind, .packages=c("deSolve")) %dorng% {
      out <- totcases.nstrain.vaxfrac(eta,R0base.vector[i],vaxfrac.vector[j],
                                          vaxfrac.vector[j],evo.is,evo.is1,evo.is2)
      tempvec<-matrix(0,1,7)
      tempvec[1,1]<-out$tot
      tempvec[1,2]<-out$sev
      tempvec[1,3]<-out$tota
      tempvec[1,4]<-out$seva
      tempvec[1,5]<-out$totb
      tempvec[1,6]<-out$sevb
      tempvec[1,7]<-out$strain
      
      tempvec
    }
    stopCluster(cl)
    #toc()
    
    output.totcases.bv.R0[i,j] <- mean(randvec[,1])
    output.sevcases.bv.R0[i,j] <- mean(randvec[,2])
    output.totcases.a.bv.R0[i,j] <- mean(randvec[,3])
    output.sevcases.a.bv.R0[i,j] <- mean(randvec[,4])
    output.totcases.b.bv.R0[i,j] <- mean(randvec[,5])
    output.sevcases.b.bv.R0[i,j] <- mean(randvec[,6])
    output.nstrain.bv.R0[i,j] <- mean(randvec[,7])
  }
}
store.output.totcases.bv.R0 <-output.totcases.bv.R0
store.output.sevcases.bv.R0 <-output.sevcases.bv.R0
store.output.totcases.a.bv.R0 <-output.totcases.a.bv.R0
store.output.sevcases.a.bv.R0 <-output.sevcases.a.bv.R0
store.output.totcases.b.bv.R0 <-output.totcases.b.bv.R0
store.output.sevcases.b.bv.R0 <-output.sevcases.b.bv.R0
store.output.nstrain.bv.R0<-output.nstrain.bv.R0

####Normalize Data
###Natural Infections Worse for Evolution
#Vary Immigration
output.totcases.bni.imm <- output.totcases.bni.imm / max(store.output.totcases.bni.imm)
output.sevcases.bni.imm <- output.sevcases.bni.imm / max(store.output.sevcases.bni.imm)
output.totcases.a.bni.imm <- output.totcases.a.bni.imm / max(store.output.totcases.a.bni.imm)
output.sevcases.a.bni.imm <- output.sevcases.a.bni.imm / max(store.output.sevcases.a.bni.imm)
output.totcases.b.bni.imm <- output.totcases.b.bni.imm / max(store.output.totcases.b.bni.imm)
output.sevcases.b.bni.imm <- output.sevcases.b.bni.imm / max(store.output.sevcases.b.bni.imm)
output.nstrain.bni.imm <- output.nstrain.bni.imm / max(store.output.nstrain.bni.imm)
#Vary R0
output.totcases.bni.R0 <- output.totcases.bni.R0 / max(store.output.totcases.bni.R0)
output.sevcases.bni.R0 <- output.sevcases.bni.R0 / max(store.output.sevcases.bni.R0)
output.totcases.a.bni.R0 <- output.totcases.a.bni.R0 / max(store.output.totcases.a.bni.R0)
output.sevcases.a.bni.R0 <- output.sevcases.a.bni.R0 / max(store.output.sevcases.a.bni.R0)
output.totcases.b.bni.R0 <- output.totcases.b.bni.R0 / max(store.output.totcases.b.bni.R0)
output.sevcases.b.bni.R0 <- output.sevcases.b.bni.R0 / max(store.output.sevcases.b.bni.R0)
output.nstrain.bni.R0 <- output.nstrain.bni.R0 / max(store.output.nstrain.bni.R0)

###Infections After Waned Vaccine Worse for Evolution
#Vary Immigration
output.totcases.bv.imm <- output.totcases.bv.imm / max(store.output.totcases.bv.imm)
output.sevcases.bv.imm <- output.sevcases.bv.imm / max(store.output.sevcases.bv.imm)
output.totcases.a.bv.imm <- output.totcases.a.bv.imm / max(store.output.totcases.a.bv.imm)
output.sevcases.a.bv.imm <- output.sevcases.a.bv.imm / max(store.output.sevcases.a.bv.imm)
output.totcases.b.bv.imm <- output.totcases.b.bv.imm / max(store.output.totcases.b.bv.imm)
output.sevcases.b.bv.imm <- output.sevcases.b.bv.imm / max(store.output.sevcases.b.bv.imm)
output.nstrain.bv.imm <- output.nstrain.bv.imm / max(store.output.nstrain.bv.imm)
#One Dose
output.totcases.bv.R0 <- output.totcases.bv.R0 /max(store.output.totcases.bv.R0)
output.sevcases.bv.R0 <- output.sevcases.bv.R0 /max(store.output.sevcases.bv.R0)
output.totcases.a.bv.R0 <- output.totcases.a.bv.R0 /max(store.output.totcases.a.bv.R0)
output.sevcases.a.bv.R0 <- output.sevcases.a.bv.R0 /max(store.output.sevcases.a.bv.R0)
output.totcases.b.bv.R0 <- output.totcases.b.bv.R0 /max(store.output.totcases.b.bv.R0)
output.sevcases.b.bv.R0 <- output.sevcases.b.bv.R0 /max(store.output.sevcases.b.bv.R0)
output.nstrain.bv.R0 <- output.nstrain.bv.R0 / max(store.output.nstrain.bv.R0)

####Melt Data for Plotting
###Natural Infections Worse for Evolution
##Immigration
output.totcases.bni.imm.melt <- melt(output.totcases.bni.imm)
output.totcases.bni.imm.melt$Var1 <- eta.vector[output.totcases.bni.imm.melt$Var1]
output.totcases.bni.imm.melt$Var2 <- vaxfrac.vector[output.totcases.bni.imm.melt$Var2]
#
output.sevcases.bni.imm.melt <- melt(output.sevcases.bni.imm)
output.sevcases.bni.imm.melt$Var1 <- eta.vector[output.sevcases.bni.imm.melt$Var1]
output.sevcases.bni.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.bni.imm.melt$Var2]
#
output.totcases.a.bni.imm.melt <- melt(output.totcases.a.bni.imm)
output.totcases.a.bni.imm.melt$Var1 <- eta.vector[output.totcases.a.bni.imm.melt$Var1]
output.totcases.a.bni.imm.melt$Var2 <- vaxfrac.vector[output.totcases.a.bni.imm.melt$Var2]
#
output.sevcases.a.bni.imm.melt <- melt(output.sevcases.a.bni.imm)
output.sevcases.a.bni.imm.melt$Var1 <- eta.vector[output.sevcases.a.bni.imm.melt$Var1]
output.sevcases.a.bni.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.a.bni.imm.melt$Var2]
#
output.totcases.b.bni.imm.melt <- melt(output.totcases.b.bni.imm)
output.totcases.b.bni.imm.melt$Var1 <- eta.vector[output.totcases.b.bni.imm.melt$Var1]
output.totcases.b.bni.imm.melt$Var2 <- vaxfrac.vector[output.totcases.b.bni.imm.melt$Var2]
#
output.sevcases.b.bni.imm.melt <- melt(output.sevcases.b.bni.imm)
output.sevcases.b.bni.imm.melt$Var1 <- eta.vector[output.sevcases.b.bni.imm.melt$Var1]
output.sevcases.b.bni.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.b.bni.imm.melt$Var2]
#
output.nstrain.bni.imm.melt <- melt(output.nstrain.bni.imm)
output.nstrain.bni.imm.melt$Var1 <- eta.vector[output.nstrain.bni.imm.melt$Var1]
output.nstrain.bni.imm.melt$Var2 <- vaxfrac.vector[output.nstrain.bni.imm.melt$Var2]
##R0
output.totcases.bni.R0.melt <- melt(output.totcases.bni.R0)
output.totcases.bni.R0.melt$Var1 <- R0base.vector[output.totcases.bni.R0.melt$Var1]
output.totcases.bni.R0.melt$Var2 <- vaxfrac.vector[output.totcases.bni.R0.melt$Var2]
#
output.sevcases.bni.R0.melt <- melt(output.sevcases.bni.R0)
output.sevcases.bni.R0.melt$Var1 <- R0base.vector[output.sevcases.bni.R0.melt$Var1]
output.sevcases.bni.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.bni.R0.melt$Var2]
#
output.totcases.a.bni.R0.melt <- melt(output.totcases.a.bni.R0)
output.totcases.a.bni.R0.melt$Var1 <- R0base.vector[output.totcases.a.bni.R0.melt$Var1]
output.totcases.a.bni.R0.melt$Var2 <- vaxfrac.vector[output.totcases.a.bni.R0.melt$Var2]
#
output.sevcases.a.bni.R0.melt <- melt(output.sevcases.a.bni.R0)
output.sevcases.a.bni.R0.melt$Var1 <- R0base.vector[output.sevcases.a.bni.R0.melt$Var1]
output.sevcases.a.bni.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.a.bni.R0.melt$Var2]
#
output.totcases.b.bni.R0.melt <- melt(output.totcases.b.bni.R0)
output.totcases.b.bni.R0.melt$Var1 <- R0base.vector[output.totcases.b.bni.R0.melt$Var1]
output.totcases.b.bni.R0.melt$Var2 <- vaxfrac.vector[output.totcases.b.bni.R0.melt$Var2]
#
output.sevcases.b.bni.R0.melt <- melt(output.sevcases.b.bni.R0)
output.sevcases.b.bni.R0.melt$Var1 <- R0base.vector[output.sevcases.b.bni.R0.melt$Var1]
output.sevcases.b.bni.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.b.bni.R0.melt$Var2]
#
output.nstrain.bni.R0.melt <- melt(output.nstrain.bni.R0)
output.nstrain.bni.R0.melt$Var1 <- R0base.vector[output.nstrain.bni.R0.melt$Var1]
output.nstrain.bni.R0.melt$Var2 <- vaxfrac.vector[output.nstrain.bni.R0.melt$Var2]
###Infections After Waned Vaccine Worse for Evolution
##Immigration
output.totcases.bv.imm.melt <- melt(output.totcases.bv.imm)
output.totcases.bv.imm.melt$Var1 <- eta.vector[output.totcases.bv.imm.melt$Var1]
output.totcases.bv.imm.melt$Var2 <- vaxfrac.vector[output.totcases.bv.imm.melt$Var2]
#
output.sevcases.bv.imm.melt <- melt(output.sevcases.bv.imm)
output.sevcases.bv.imm.melt$Var1 <- eta.vector[output.sevcases.bv.imm.melt$Var1]
output.sevcases.bv.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.bv.imm.melt$Var2]
#
output.totcases.a.bv.imm.melt <- melt(output.totcases.a.bv.imm)
output.totcases.a.bv.imm.melt$Var1 <- eta.vector[output.totcases.a.bv.imm.melt$Var1]
output.totcases.a.bv.imm.melt$Var2 <- vaxfrac.vector[output.totcases.a.bv.imm.melt$Var2]
#
output.sevcases.a.bv.imm.melt <- melt(output.sevcases.a.bv.imm)
output.sevcases.a.bv.imm.melt$Var1 <- eta.vector[output.sevcases.a.bv.imm.melt$Var1]
output.sevcases.a.bv.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.a.bv.imm.melt$Var2]
#
output.totcases.b.bv.imm.melt <- melt(output.totcases.b.bv.imm)
output.totcases.b.bv.imm.melt$Var1 <- eta.vector[output.totcases.b.bv.imm.melt$Var1]
output.totcases.b.bv.imm.melt$Var2 <- vaxfrac.vector[output.totcases.b.bv.imm.melt$Var2]
#
output.sevcases.b.bv.imm.melt <- melt(output.sevcases.b.bv.imm)
output.sevcases.b.bv.imm.melt$Var1 <- eta.vector[output.sevcases.b.bv.imm.melt$Var1]
output.sevcases.b.bv.imm.melt$Var2 <- vaxfrac.vector[output.sevcases.b.bv.imm.melt$Var2]
#
output.nstrain.bv.imm.melt <- melt(output.nstrain.bv.imm)
output.nstrain.bv.imm.melt$Var1 <- eta.vector[output.nstrain.bv.imm.melt$Var1]
output.nstrain.bv.imm.melt$Var2 <- vaxfrac.vector[output.nstrain.bv.imm.melt$Var2]
##R0
output.totcases.bv.R0.melt <- melt(output.totcases.bv.R0)
output.totcases.bv.R0.melt$Var1 <- R0base.vector[output.totcases.bv.R0.melt$Var1]
output.totcases.bv.R0.melt$Var2 <- vaxfrac.vector[output.totcases.bv.R0.melt$Var2]
#
output.sevcases.bv.R0.melt <- melt(output.sevcases.bv.R0)
output.sevcases.bv.R0.melt$Var1 <- R0base.vector[output.sevcases.bv.R0.melt$Var1]
output.sevcases.bv.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.bv.R0.melt$Var2]
#
output.totcases.a.bv.R0.melt <- melt(output.totcases.a.bv.R0)
output.totcases.a.bv.R0.melt$Var1 <- R0base.vector[output.totcases.a.bv.R0.melt$Var1]
output.totcases.a.bv.R0.melt$Var2 <- vaxfrac.vector[output.totcases.a.bv.R0.melt$Var2]
#
output.sevcases.a.bv.R0.melt <- melt(output.sevcases.a.bv.R0)
output.sevcases.a.bv.R0.melt$Var1 <- R0base.vector[output.sevcases.a.bv.R0.melt$Var1]
output.sevcases.a.bv.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.a.bv.R0.melt$Var2]
#
output.totcases.b.bv.R0.melt <- melt(output.totcases.b.bv.R0)
output.totcases.b.bv.R0.melt$Var1 <- R0base.vector[output.totcases.b.bv.R0.melt$Var1]
output.totcases.b.bv.R0.melt$Var2 <- vaxfrac.vector[output.totcases.b.bv.R0.melt$Var2]
#
output.sevcases.b.bv.R0.melt <- melt(output.sevcases.b.bv.R0)
output.sevcases.b.bv.R0.melt$Var1 <- R0base.vector[output.sevcases.b.bv.R0.melt$Var1]
output.sevcases.b.bv.R0.melt$Var2 <- vaxfrac.vector[output.sevcases.b.bv.R0.melt$Var2]
#
output.nstrain.bv.R0.melt <- melt(output.nstrain.bv.R0)
output.nstrain.bv.R0.melt$Var1 <- R0base.vector[output.nstrain.bv.R0.melt$Var1]
output.nstrain.bv.R0.melt$Var2 <- vaxfrac.vector[output.nstrain.bv.R0.melt$Var2]


####Combine Data for Plotting
#Natural Infections Worse for Evolution
combined.data.bni.imm=bind_rows(list("Cases in HAR"=output.totcases.a.bni.imm.melt,
                                     "Sev. cases in HAR"=output.sevcases.a.bni.imm.melt,
                                     "Cases in LAR"=output.totcases.b.bni.imm.melt,
                                     "Sev. cases in LAR"=output.sevcases.b.bni.imm.melt,
                                     "Cases (Total)"=output.totcases.bni.imm.melt,
                                     "Sev. cases (Total)"=output.sevcases.bni.imm.melt,
                                     "PTIs"=output.nstrain.bni.imm.melt),.id="val")
combined.data.bni.R0=bind_rows(list( "Cases in HAR"=output.totcases.a.bni.R0.melt,
                                     "Sev. cases in HAR"=output.sevcases.a.bni.R0.melt,
                                     "Cases in LAR"=output.totcases.b.bni.R0.melt,
                                     "Sev. cases in LAR"=output.sevcases.b.bni.R0.melt,
                                     "Cases (Total)"=output.totcases.bni.R0.melt,
                                     "Sev. cases (Total)"=output.sevcases.bni.R0.melt,
                                     "PTIs"=output.nstrain.bni.R0.melt),.id="val")

#Infections After Waned Vaccine Worse for Evolution
combined.data.bv.imm=bind_rows(list( "Cases in HAR"=output.totcases.a.bv.imm.melt,
                                     "Sev. cases in HAR"=output.sevcases.a.bv.imm.melt,
                                     "Cases in LAR"=output.totcases.b.bv.imm.melt,
                                     "Sev. cases in LAR"=output.sevcases.b.bv.imm.melt,
                                     "Cases (Total)"=output.totcases.bv.imm.melt,
                                     "Sev. cases (Total)"=output.sevcases.bv.imm.melt,
                                     "PTIs"=output.nstrain.bv.imm.melt),.id="val")
combined.data.bv.R0=bind_rows(list( "Cases in HAR"=output.totcases.a.bv.R0.melt,
                                    "Sev. cases in HAR"=output.sevcases.a.bv.R0.melt,
                                    "Cases in LAR"=output.totcases.b.bv.R0.melt,
                                    "Sev. cases in LAR"=output.sevcases.b.bv.R0.melt,
                                    "Cases (Total)"=output.totcases.bv.R0.melt,
                                    "Sev. cases (Total)"=output.sevcases.bv.R0.melt,
                                    "PTIs"=output.nstrain.bv.R0.melt),.id="val")

#Specify facet order for plotting
combined.data.bni.imm$val_f = factor(combined.data.bni.imm$val, levels=c('Cases (Total)','Sev. cases (Total)',
                                                                         'Cases in HAR','Sev. cases in HAR',
                                                                         'Cases in LAR', 'Sev. cases in LAR',
                                                                         'PTIs'))
combined.data.bni.R0$val_f = factor(combined.data.bni.R0$val, levels=c('Cases (Total)','Sev. cases (Total)',
                                                                       'Cases in HAR','Sev. cases in HAR',
                                                                       'Cases in LAR', 'Sev. cases in LAR',
                                                                       'PTIs'))
combined.data.bv.imm$val_f = factor(combined.data.bv.imm$val, levels=c('Cases (Total)','Sev. cases (Total)',
                                                                       'Cases in HAR','Sev. cases in HAR',
                                                                       'Cases in LAR', 'Sev. cases in LAR',
                                                                       'PTIs'))
combined.data.bv.R0$val_f = factor(combined.data.bv.R0$val, levels=c('Cases (Total)','Sev. cases (Total)',
                                                                     'Cases in HAR','Sev. cases in HAR',
                                                                     'Cases in LAR', 'Sev. cases in LAR',
                                                                     'PTIs'))


####Plotting
#Natural Infections Worse for Evolution
#Immigration
grid.plot.bni.imm=ggplot(data=combined.data.bni.imm)+geom_tile(aes(y=Var1,x=Var2,fill=value))+
  facet_grid(~val_f)+scale_fill_binned(name="Normalized value",breaks=seq(0.1,0.9,by=0.1))+
  xlab("Vaccine fraction retained by HAR")+ylab("Immigration rate") + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 10 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 10 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 10,face="bold") 
  )

#pdf("BadNatInf_Immigration_April82021.pdf",width=8,height=6)
#print(grid.plot.bni.imm)
#dev.off()

#R0 
grid.plot.bni.R0=ggplot(data=combined.data.bni.R0)+geom_tile(aes(y=Var1,x=Var2,fill=value))+
  facet_grid(~val_f)+scale_fill_binned(name="Normalized value",breaks=seq(0.1,0.9,by=0.1))+
  xlab("Vaccine fraction retained by HAR")+ylab("Reproduction number ratio") + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 10 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 10 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 10,face="bold") 
  )

#pdf("BadNatInf_R0_April82021.pdf",width=8,height=6)
#print(grid.plot.bni.R0)
#dev.off()

grid.plot.bni <- plot_grid(grid.plot.bni.imm, grid.plot.bni.R0, labels=c("A","B"),ncol=1,align="v", axis="l")
pdf("BadNatInf_withSev_Nb2Na0p5_nu0p02_May272021.pdf",width=12,height=6)
print(grid.plot.bni)
dev.off()

#Infections After Waned Vaccine Worse for Evolution
#Immigration
grid.plot.bv.imm=ggplot(data=combined.data.bv.imm)+geom_tile(aes(y=Var1,x=Var2,fill=value))+
  facet_grid(~val_f)+scale_fill_binned(name="Normalized value",breaks=seq(0.1,0.9,by=0.1))+
  xlab("Vaccine fraction retained by HAR")+ylab("Immigration rate") + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 10 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 10 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 10,face="bold") 
  )

#pdf("BadVax_Immigration_April82021.pdf",width=8,height=6)
#print(grid.plot.bv.imm)
#dev.off()

#R0 
grid.plot.bv.R0=ggplot(data=combined.data.bv.R0)+geom_tile(aes(y=Var1,x=Var2,fill=value))+
  facet_grid(~val_f)+scale_fill_binned(name="Normalized value",breaks=seq(0.1,0.9,by=0.1))+
  xlab("Vaccine fraction retained by HAR")+ylab("Reproduction number ratio") + theme_bw()+
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        strip.background = element_blank(),
        panel.border = element_rect(colour = "black", fill = NA,),
        axis.text = element_text( size = 10 ),
        axis.title = element_text( size = 12),
        strip.text = element_text(size = 10 ,face="bold"),
        legend.text = element_text(size = 10),
        legend.title = element_text(size = 10,face="bold") 
  )

#pdf("BadVax_R0_April82021.pdf",width=8,height=6)
#print(grid.plot.bv.R0)
#dev.off()

grid.plot.bv <- plot_grid(grid.plot.bv.imm, grid.plot.bv.R0, labels=c("C","D"),ncol=1,align="v", axis="l")
pdf("BadVax_withSev_Nb2Na0p5_nu0p02_May272021.pdf",width=12,height=6)
print(grid.plot.bv)
dev.off()


####Reset
# output.totcases.bni.imm<-store.output.totcases.bni.imm
# output.sevcases.bni.imm<-store.output.sevcases.bni.imm
# output.totcases.a.bni.imm<-store.output.totcases.a.bni.imm
# output.sevcases.a.bni.imm<-store.output.sevcases.a.bni.imm
# output.totcases.b.bni.imm<-store.output.totcases.b.bni.imm
# output.sevcases.b.bni.imm<-store.output.sevcases.b.bni.imm
# output.nstrain.bni.imm<-store.output.nstrain.bni.imm
# output.totcases.bni.R0<-store.output.totcases.bni.R0
# output.sevcases.bni.R0<-store.output.sevcases.bni.R0
# output.totcases.a.bni.R0<-store.output.totcases.a.bni.R0
# output.sevcases.a.bni.R0<-store.output.sevcases.a.bni.R0
# output.totcases.b.bni.R0<-store.output.totcases.b.bni.R0
# output.sevcases.b.bni.R0<-store.output.sevcases.b.bni.R0
# output.nstrain.bni.R0<-store.output.nstrain.bni.R0
# output.totcases.bv.imm<-store.output.totcases.bv.imm
# output.sevcases.bv.imm<-store.output.sevcases.bv.imm
# output.totcases.a.bv.imm<-store.output.totcases.a.bv.imm
# output.sevcases.a.bv.imm<-store.output.sevcases.a.bv.imm
# output.totcases.b.bv.imm<-store.output.totcases.b.bv.imm
# output.sevcases.b.bv.imm<-store.output.sevcases.b.bv.imm
# output.nstrain.bv.imm<-store.output.nstrain.bv.imm
# output.totcases.bv.R0<-store.output.totcases.bv.R0
# output.sevcases.bv.R0<-store.output.sevcases.bv.R0
# output.totcases.a.bv.R0<-store.output.totcases.a.bv.R0
# output.sevcases.a.bv.R0<-store.output.sevcases.a.bv.R0
# output.totcases.b.bv.R0<-store.output.totcases.b.bv.R0
# output.sevcases.b.bv.R0<-store.output.sevcases.b.bv.R0
# output.nstrain.bv.R0<-store.output.nstrain.bv.R0
