library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)

sir_s_.model.doses.switch<-function (t,x,parameters) {
  
  SP=x[1]
  IP=x[2]
  R=x[3]
  SS=x[4]
  IS=x[5]
  V1=x[6]
  V2=x[7]
  IV=x[8]
  SS1=x[9]
  SS2=x[10]
  IS1=x[11]
  IS2=x[12]
  
  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  alphaV=parameters[["alphaV"]]
  alpha1=parameters[["alpha1"]]
  alpha2=parameters[["alpha2"]]
  d=parameters[["d"]]
  epsilon=parameters[["epsilon"]]
  epsilonV1=parameters[["epsilonV1"]]
  epsilonV2=parameters[["epsilonV2"]]
  epsilon1=parameters[["epsilon1"]]
  epsilon2=parameters[["epsilon2"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  R0red.increase=parameters[["R0red.increase"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  tred3=parameters[["tred3"]]
  tred4=parameters[["tred4"]]
  #Vaccination
  nu.init=parameters[["nu.init"]]
  omega.init=parameters[["omega.init"]]
  nu.switch=parameters[["nu.switch"]]
  omega.switch=parameters[["omega.switch"]]
  t.switch=parameters[["t.switch"]]
  rho1=parameters[["rho1"]]
  rho2=parameters[["rho2"]]
  tvax=parameters[["tvax"]]
  
  if((t > tred1) & (t < tred2)){
    beta = beta * R0red
  }
  if((t >= tred2) & (t < tred3)){
    beta = beta * R0red
  }
  if((t >= tred3) & (t < tred4)){
    beta = beta * R0red * R0red.increase
  }
  svax=0
  if(t>tvax){
    svax=1
  }
  if(t>t.switch){
    omega=omega.switch
    nu=nu.switch
  } else {
    omega=omega.init
    nu=nu.init
  }
  #Equations
  dSPdt=mu-(beta*SP*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2))-mu*SP-svax*nu*SP
  dIPdt=(beta*SP*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2))-(gamma+mu)*IP
  dRdt=gamma*(IP+IS+IV+IS1+IS2)-(delta+mu)*R
  dSSdt=delta*R-epsilon*(beta*SS*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2))-mu*SS-svax*nu*SS
  dISdt=epsilon*(beta*SS*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2))-(gamma+mu)*IS
  dV1dt=svax*nu*(SP+d*SS)-epsilonV1*beta*V1*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-(omega+rho1+mu)*V1
  dV2dt=(1-d)*svax*nu*SS+omega*V1-epsilonV2*beta*V2*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-(rho2+mu)*V2
  dIVdt=(epsilonV1*beta*V1+epsilonV2*beta*V2)*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-(gamma+mu)*IV
  dSS1dt=rho1*V1-epsilon1*beta*SS1*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-mu*SS1
  dSS2dt=rho2*V2-epsilon2*beta*SS2*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-mu*SS2
  dIS1dt=epsilon1*beta*SS1*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-(gamma+mu)*IS1
  dIS2dt=epsilon2*beta*SS2*(IP+alpha*IS+alphaV*IV+alpha1*IS1+alpha2*IS2)-(gamma+mu)*IS2
  dxdt <- c(dSPdt,dIPdt,dRdt,dSSdt,dISdt,dV1dt,dV2dt,dIVdt,dSS1dt,dSS2dt,dIS1dt,dIS2dt)
  list(dxdt)
}

plots.doses.switch=function(alpha,tred1,tred2,tred3,tred4,delta,rho2,tvax,
                            nuVector,primary.burden,secondary.burden,epsilon,
                            epsilonV1,epsilonV2,epsilon1,epsilon2,omega.init,rho1,
                            one.dose.burden,two.doses.burden,vaccine.burden,d,
                            start.value,max.y.cases,t.switch,nu.switch.vector,omega.switch,
                            weight.IS,weight.IS1,weight.IS2,
                            weight2.IS,weight2.IS1,weight2.IS2,
                            weight3.IS,weight3.IS1,weight3.IS2,
                            weight4.IS,weight4.IS1,weight4.IS2){
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  valuesV1=list()
  valuesV2=list()
  valuesIV=list()
  valuesSS1=list()
  valuesSS2=list()
  valuesIS1=list()
  valuesIS2=list()
  values.weights1=list()
  values.weights2=list()
  values.weights3=list()
  values.weights4=list()
  
  plots=list()
  plots.SS=list()
  plots.IS=list()
  plots.IS.weights=list()
  data.burden.list=list()
  N=1
  mu=1/(50*52)
  gamma=7/5
  R0red=0.45
  R0red.increase=1.3
  alphaV=alpha
  alpha1=alpha
  alpha2=alpha
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=2.3*R0.listOrig$hku1/(mean(R0.listOrig$hku1))
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  ss1.col=rgb(1,0.5,0)
  ss2.col=rgb(1,0.9,0)
  is.col=rgb(1,0.81,0.85)
  v2.col=rgb(0.33,0.51,0.21)
  v1.col=rgb(0.33,0.81,0.21)
  iv.col=rgb(0.6,0.4,0.4)
  is2.col=rgb(1,0.6,1)
  is1.col=rgb(1,0.1,0.5)
  xstart=c(SP=1-I0,IP=I0,R=0,SS=0,IS=0,V1=0,V2=0,IV=0,SS1=0,SS2=0,IS1=0,IS2=0) 
  for  (i in  1:length(nuVector)){
    nu.init=nuVector[i]
    nu.switch=nu.switch.vector[i]
    parameters=list(N=N,mu=mu,gamma=gamma,alpha=alpha,
                    epsilon=epsilon,delta=delta,
                    R0.list=R0.list,R0red=R0red,R0red.increase=R0red.increase,
                    tred1=tred1,tred2=tred2,tred3=tred3,tred4=tred4,
                    tvax=tvax,nu.init=nu.init,nu.switch=nu.switch,rho1=rho1,rho2=rho2,
                    epsilonV1=epsilonV1,epsilonV2=epsilonV2,
                    epsilon1=epsilon1,epsilon2=epsilon2,alphaV=alphaV,alpha1=alpha1,alpha2=alpha2,
                    omega.init=omega.init,omega.switch=omega.switch,d=d,t.switch=t.switch)
    out=as.data.frame(rk(func=sir_s_.model.doses.switch,y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP[start.value:(5*52)]
    valuesIS[[i]]=out$IS[start.value:(5*52)]
    valuesR[[i]]=out$R[start.value:(5*52)]
    valuesSS[[i]]=out$SS[start.value:(5*52)]
    valuesSP[[i]]=out$SP[start.value:(5*52)]
    valuesV1[[i]]=out$V1[start.value:(5*52)]
    valuesV2[[i]]=out$V2[start.value:(5*52)]
    valuesSS1[[i]]=out$SS1[start.value:(5*52)]
    valuesSS2[[i]]=out$SS2[start.value:(5*52)]
    valuesIS1[[i]]=out$IS1[start.value:(5*52)]
    valuesIS2[[i]]=out$IS2[start.value:(5*52)]
    valuesIV[[i]]=out$IV[start.value:(5*52)]
    forAreaPlotIP=valuesIP[[i]]
    forAreaPlotIS=valuesIS[[i]]
    forAreaPlotR=valuesR[[i]]
    forAreaPlotSS=valuesSS[[i]]
    forAreaPlotSP=valuesSP[[i]]
    forAreaPlotV1=valuesV1[[i]]
    forAreaPlotV2=valuesV2[[i]]
    forAreaPlotIV=valuesIV[[i]]
    forAreaPlotSS1=valuesSS1[[i]]
    forAreaPlotSS2=valuesSS2[[i]]
    forAreaPlotIS1=valuesIS1[[i]]
    forAreaPlotIS2=valuesIS2[[i]]
    scalar=5
    all.cases=forAreaPlotIP+forAreaPlotIS+forAreaPlotIV+forAreaPlotIS1+forAreaPlotIS2
    burden=primary.burden*forAreaPlotIP+secondary.burden*forAreaPlotIS+
      vaccine.burden*forAreaPlotIV+one.dose.burden*forAreaPlotIS1+two.doses.burden*forAreaPlotIS2
    data.burden=data.frame(xx=times[start.value:(5*52)],yy=burden,zz=all.cases)
    data.burden.list[[i]]=data.burden
    
    data.AP.IP=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotIP)
    data.AP.IS=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotIS)
    data.AP.R=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotR)
    data.AP.SS=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotSS)
    data.AP.SP=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotSP)
    data.AP.V1=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotV1)
    data.AP.V2=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotV2)
    data.AP.IV=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotIV)
    data.AP.SS1=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotSS1)
    data.AP.SS2=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotSS2)
    data.AP.IS1=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotIS1)
    data.AP.IS2=data.frame(xx=times[start.value:(5*52)],yy=forAreaPlotIS2)
    
    data.AP.list=list("primary infection"=data.AP.IP,"secondary infection"=data.AP.IS,
                      "fully immune"=data.AP.R,"partially immune"=data.AP.SS,
                      "fully susceptible"=data.AP.SP,
                      "one dose"=data.AP.V1, "two doses"=data.AP.V2,
                      "one dose, waned"=data.AP.SS1, "two doses, waned"=data.AP.SS2,
                      "infection after vaccine"=data.AP.IV,"infection after one dose waning"=data.AP.IS1,
                      "infection after two doses waning"=data.AP.IS2)
    
    area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
    area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("one dose","two doses","fully susceptible","fully immune",
                                                                                    "partially immune","one dose, waned","two doses, waned",
                                                                                    "infection after vaccine","infection after one dose waning",
                                                                                    "infection after two doses waning","secondary infection",
                                                                                    "primary infection"))
    plots[[i]]=area.plots.timecourses
    
    data.IS.weighted.sum=data.frame(xx=times[start.value:(5*52)],
                                    yy=(weight.IS*forAreaPlotIS+weight.IS1*forAreaPlotIS1+weight.IS2*forAreaPlotIS2))
    data.IS.weighted2.sum=data.frame(xx=times[start.value:(5*52)],
                                     yy=(weight2.IS*forAreaPlotIS+weight2.IS1*forAreaPlotIS1+weight2.IS2*forAreaPlotIS2))
    data.IS.weighted3.sum=data.frame(xx=times[start.value:(5*52)],
                                     yy=(weight3.IS*forAreaPlotIS+weight3.IS1*forAreaPlotIS1+weight3.IS2*forAreaPlotIS2))
    data.IS.weighted4.sum=data.frame(xx=times[start.value:(5*52)],
                                     yy=(weight4.IS*forAreaPlotIS+weight4.IS1*forAreaPlotIS1+weight4.IS2*forAreaPlotIS2))
    
    values.weights1[[i]]=data.IS.weighted.sum$yy
    values.weights2[[i]]=data.IS.weighted2.sum$yy
    values.weights3[[i]]=data.IS.weighted3.sum$yy
    values.weights4[[i]]=data.IS.weighted4.sum$yy
    
    data.SS.list=list("nat. infection waned"=data.AP.SS,
                      "one dose, waned"=data.AP.SS1,
                      "two doses, waned"=data.AP.SS2)
    
    data.IS.list=list("nat. infection waned"=data.AP.IS,
                      "one dose, waned"=data.AP.IS1,
                      "two doses, waned"=data.AP.IS2)
    
    data.IS.weights.list=list("scenario I"=data.IS.weighted.sum,
                              "scenario II"=data.IS.weighted2.sum,
                              "scenario III"=data.IS.weighted3.sum,
                              "scenario IV"=data.IS.weighted4.sum)
    
    SS.timecourses=bind_rows(data.SS.list,.id=c("immunity"))
    IS.timecourses=bind_rows(data.IS.list,.id=c("immunity"))
    IS.weights.timecourses=bind_rows(data.IS.weights.list,.id=c("immunity"))
    SS.timecourses$immunity=factor(SS.timecourses$immunity,
                                   levels=c("nat. infection waned","one dose, waned","two doses, waned"))
    IS.timecourses$immunity=factor(IS.timecourses$immunity,
                                   levels=c("nat. infection waned","one dose, waned","two doses, waned"))
    IS.weights.timecourses$immunity=factor(IS.weights.timecourses$immunity,
                                           levels=c("scenario I","scenario II","scenario III","scenario IV"))
    
    plots.SS[[i]]=SS.timecourses
    plots.IS[[i]]=IS.timecourses
    plots.IS.weights[[i]]=IS.weights.timecourses
  }
  names(plots)=c("HAR","LAR")
  area.plots.timecourses.combined=bind_rows(plots,.id="country")
  
  names(data.burden.list)=c("HAR","LAR")
  data.burden.combined=bind_rows(data.burden.list,.id="country")
  
  names(plots.SS)=c("HAR","LAR")
  SS.timecourses.combined=bind_rows(plots.SS,.id="country")
  
  names(plots.IS)=c("HAR","LAR")
  IS.timecourses.combined=bind_rows(plots.IS,.id="country")
  
  names(plots.IS.weights)=c("HAR","LAR")
  IS.weights.timecourses.combined=bind_rows(plots.IS.weights,.id="country")
  
  total.weights1=values.weights1[[1]]+values.weights1[[2]]
  total.weights2=values.weights2[[1]]+values.weights2[[2]]
  total.weights3=values.weights3[[1]]+values.weights3[[2]]
  total.weights4=values.weights4[[1]]+values.weights4[[2]]
  data.total.weights1=data.frame(xx=times[start.value:(5*52)],yy=total.weights1)
  data.total.weights2=data.frame(xx=times[start.value:(5*52)],yy=total.weights2)
  data.total.weights3=data.frame(xx=times[start.value:(5*52)],yy=total.weights3)
  data.total.weights4=data.frame(xx=times[start.value:(5*52)],yy=total.weights4)
  
  total.IS.weights.list=list("scenario I"=data.total.weights1,
                             "scenario II"=data.total.weights2,
                             "scenario III"=data.total.weights3,
                             "scenario IV"=data.total.weights4)
  
  total.IS.weights.timecourses=bind_rows(total.IS.weights.list,.id=c("immunity"))
  total.IS.weights.timecourses$immunity=factor(total.IS.weights.timecourses$immunity,
                                               levels=c("scenario I","scenario II",
                                                        "scenario III","scenario IV"))
  
  plot.total.evo=ggplot(total.IS.weights.timecourses,aes(x=xx/52,y=yy,colour=immunity))+
    geom_line(size=1.2)+
    scale_y_continuous(name="",limits=c(0,0.06))+
    scale_x_continuous(name="Time (years)",expand=c(0,0))+
    scale_color_manual(name="",values=c("scenario I"="black","scenario II"="blue",
                                        "scenario III"="purple","scenario IV"="green4"))+
    theme(legend.position="none")+
    theme(legend.key.width=unit(1.2,"cm"))+theme(plot.margin=unit(c(0.1,0.1,0.1,0.1),"cm"))
  
  plot.top=ggplot(data=data.burden.combined,aes(x=xx/52))+geom_line(aes(y=zz),color="red")+
    geom_line(aes(y=yy*scalar),linetype="dashed")+facet_wrap(~country,ncol=2)+
    scale_y_continuous(name="Cases",
                       sec.axis=sec_axis(~./scalar, name="Severe"),
                       limits=c(0,max.y.cases))+
    xlab("Time (years)")+
    scale_x_continuous(expand=c(0,0))+
    theme_classic((size=14))+
    theme(panel.grid.major.x=element_line(colour="grey66",linetype="dashed"))+
    theme(axis.line.y.left=element_line(color="red"), 
          axis.ticks.y.left=element_line(color="red"),
          axis.title.y.left=element_text(color="red"),
          axis.text.y.left=element_text(color="red"))+
    theme(plot.margin=unit(c(0,0,0.1,0),"cm"))+
    theme(axis.text.x=element_blank(),axis.title.x=element_blank(),
          strip.background=element_blank())
  plot.below=ggplot(area.plots.timecourses.combined,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
    theme_classic((size=14))+facet_wrap(~country,ncol=2)+
    scale_y_continuous(name="Fraction of population")+
    scale_x_continuous(name="Time (years)",expand=c(0,0))+
    scale_fill_manual(name="",values=c(v1.col,v2.col,sp.col,r.col,ss.col,ss1.col,ss2.col,iv.col,is1.col,is2.col,is.col,ip.col))+
    theme(legend.position="none")+theme(plot.margin=unit(c(0,0,0,0),"cm"))
  plots.combined=plot.top/plot.below+plot_layout(height=c(0.4,1))+
    theme(strip.text=element_blank(),strip.background=element_blank())
  
  plot.evo=ggplot(IS.weights.timecourses.combined,aes(x=xx/52,y=yy,colour=immunity))+
    geom_line(size=1.2)+facet_wrap(~country,ncol=2)+
    scale_y_continuous(name="",limits=c(0,0.052))+
    scale_x_continuous(expand=c(0,0))+
    scale_color_manual(name="",values=c("scenario I"="black","scenario II"="blue",
                                        "scenario III"="purple","scenario IV"="green4"))+
    theme(legend.position="none")+theme(axis.title.y=element_blank(),
                                        axis.title.x=element_blank(),
                                        axis.text.x=element_blank(),
                                        plot.title=element_text(margin=margin(0,0,0,0)))+
    theme(legend.key.width=unit(1.2,"cm"))+theme(plot.margin=unit(c(0.1,0.1,0.1,0.1),"cm"))
  
  plot.IS=ggplot(IS.timecourses.combined,aes(x=xx/52,y=yy,colour=immunity,linetype=immunity))+
    geom_line(size=1)+facet_wrap(~country,ncol=2)+
    scale_y_continuous(name="",limits=c(0,0.052))+
    scale_x_continuous(name="Time (years)",expand=c(0,0))+
    scale_color_manual(name="",values=c("nat. infection waned"=is.col,
                                        "one dose, waned"=is1.col,"two doses, waned"=is2.col))+
    scale_linetype_manual(name="",values=c("nat. infection waned"="solid",
                                           "one dose, waned"="dashed","two doses, waned"="dotdash"))+
    theme(legend.position="none")+theme(axis.title.y=element_blank(),
                                        axis.title.x=element_blank(),
                                        axis.text.x=element_blank(),
                                        strip.text=element_blank(),
                                        strip.background=element_blank(),
                                        plot.title=element_text(margin=margin(0,0,0,0)))+
    theme(legend.key.width=unit(1.2,"cm"))+theme(plot.margin=unit(c(0.1,0.1,0.1,0.1),"cm"))
  
  plot.susc=ggplot(SS.timecourses.combined,aes(x=xx/52,y=yy,colour=immunity,linetype=immunity))+
    geom_line(size=1)+facet_wrap(~country,ncol=2)+
    scale_y_continuous(name="",limits=c(0,0.7))+
    scale_x_continuous(name="Time (years)",expand=c(0,0))+
    scale_color_manual(name="",values=c("nat. infection waned"=ss.col,"one dose, waned"=ss1.col,
                                        "two doses, waned"=ss2.col))+
    scale_linetype_manual(name="",values=c("nat. infection waned"="solid","one dose, waned"="dashed",
                                           "two doses, waned"="dotdash"))+
    theme(legend.position="none")+theme(axis.title.y=element_blank(),
                                        strip.text=element_blank(),
                                        strip.background=element_blank(),
                                        plot.title=element_text(margin=margin(0,0,0,0)))+
    theme(legend.key.width=unit(1.2,"cm"))+theme(plot.margin=unit(c(0.1,0.1,0.1,0.1),"cm"))
  plots.evo.combined=plot.evo/plot.IS/plot.susc
  
  return(list(plots.combined,plots.evo.combined,plot.total.evo))
}

x.percent.y.percent.switch.one.dose=function(x.percent.vector,y.percent.vector,
                                             rho1.good,rho1.bad,
                                             epsilon1.good,epsilon1.bad,
                                             weight.IS,weight.IS1,weight.IS2,
                                             weight2.IS,weight2.IS1,weight2.IS2,
                                             weight3.IS,weight3.IS1,weight3.IS2,
                                             weight4.IS,weight4.IS1,weight4.IS2){
  output.48weeks.x.percent.y.percent.no.sharing.bad=list()
  output.48weeks.x.percent.y.percent.half.sharing.bad=list()
  output.48weeks.x.percent.y.percent.no.sharing.good=list()
  output.48weeks.x.percent.y.percent.half.sharing.good=list()
  primary.burden=0.14
  secondary.burden=0.07
  one.dose.burden.bad=0.14
  one.dose.burden.good=0
  two.doses.burden=0
  vaccine.burden=0.14
  for (i in 1:length(x.percent.vector)){
    x.percent=x.percent.vector[i]
    y.percent=y.percent.vector[i]
    output.48weeks.x.percent.y.percent.no.sharing.bad[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                         c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                         0.1,0.05,epsilon1.bad,0.7,0,rho1.bad,
                                                                         one.dose.burden.bad,two.doses.burden,vaccine.burden,0.5,
                                                                         45,0.06,60,c(y.percent,0),0,
                                                                         weight.IS,weight.IS1,weight.IS2,
                                                                         weight2.IS,weight2.IS1,weight2.IS2,
                                                                         weight3.IS,weight3.IS1,weight3.IS2,
                                                                         weight4.IS,weight4.IS1,weight4.IS2)
    
    output.48weeks.x.percent.y.percent.half.sharing.bad[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                           c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                           0.1,0.05,epsilon1.bad,0.7,0,rho1.bad,
                                                                           one.dose.burden.bad,two.doses.burden,vaccine.burden,0.5,
                                                                           45,0.06,60,c(y.percent/2,y.percent/2),0,
                                                                           weight.IS,weight.IS1,weight.IS2,
                                                                           weight2.IS,weight2.IS1,weight2.IS2,
                                                                           weight3.IS,weight3.IS1,weight3.IS2,
                                                                           weight4.IS,weight4.IS1,weight4.IS2)
    
    output.48weeks.x.percent.y.percent.no.sharing.good[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                          c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                          0.1,0.05,epsilon1.good,0.7,0,rho1.good,
                                                                          one.dose.burden.good,two.doses.burden,vaccine.burden,0.5,
                                                                          45,0.06,60,c(y.percent,0),0,
                                                                          weight.IS,weight.IS1,weight.IS2,
                                                                          weight2.IS,weight2.IS1,weight2.IS2,
                                                                          weight3.IS,weight3.IS1,weight3.IS2,
                                                                          weight4.IS,weight4.IS1,weight4.IS2)
    
    output.48weeks.x.percent.y.percent.half.sharing.good[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                            c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                            0.1,0.05,epsilon1.good,0.7,0,rho1.good,
                                                                            one.dose.burden.good,two.doses.burden,vaccine.burden,0.5,
                                                                            45,0.06,60,c(y.percent/2,y.percent/2),0,
                                                                            weight.IS,weight.IS1,weight.IS2,
                                                                            weight2.IS,weight2.IS1,weight2.IS2,
                                                                            weight3.IS,weight3.IS1,weight3.IS2,
                                                                            weight4.IS,weight4.IS1,weight4.IS2)
  }
  panels=list()
  panels[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[1]]|
    output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[1]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[1]]|
                                                                 output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[1]]))
  panels[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[1]]|
                 output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[1]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[1]]|
                                                                              output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[1]]))
  panels.evo=list()
  panels.evo[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[2]]|
                  output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[2]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[2]]|
                                                                                                          output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[2]]))
  panels.evo[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[2]]|
                  output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[2]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[2]]|
                                                                                     output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[2]]))
  panels.evo.total=list()
  panels.evo.total[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[3]]|
                            output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[3]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[3]]|
                                                                                                                    output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[3]]))
  panels.evo.total[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[3]]|
                            output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[3]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[3]]|
                                                                                               output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[3]]))
  return(list(panels,panels.evo,panels.evo.total))
}


x.percent.y.percent.switch.one.dose.switch.two.dose=function(x.percent.vector,y.percent.vector,
                                                             rho1.good,rho1.bad,
                                                             epsilon1.good,epsilon1.bad,
                                                             weight.IS,weight.IS1,weight.IS2,
                                                             weight2.IS,weight2.IS1,weight2.IS2,
                                                             weight3.IS,weight3.IS1,weight3.IS2,
                                                             weight4.IS,weight4.IS1,weight4.IS2){
  output.48weeks.x.percent.y.percent.no.sharing.bad=list()
  output.48weeks.x.percent.y.percent.half.sharing.bad=list()
  output.48weeks.x.percent.y.percent.no.sharing.good=list()
  output.48weeks.x.percent.y.percent.half.sharing.good=list()
  primary.burden=0.14
  secondary.burden=0.07
  one.dose.burden.bad=0.14
  one.dose.burden.good=0
  two.doses.burden=0
  vaccine.burden=0.14
  for (i in 1:length(x.percent.vector)){
    x.percent=x.percent.vector[i]
    y.percent=y.percent.vector[i]
    output.48weeks.x.percent.y.percent.no.sharing.bad[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                              c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                              0.1,0.05,epsilon1.bad,0.7,0,rho1.bad,
                                                                              one.dose.burden.bad,two.doses.burden,vaccine.burden,0.5,
                                                                              45,0.06,60,c(y.percent/2,0),1/4,
                                                                              weight.IS,weight.IS1,weight.IS2,
                                                                              weight2.IS,weight2.IS1,weight2.IS2,
                                                                              weight3.IS,weight3.IS1,weight3.IS2,
                                                                              weight4.IS,weight4.IS1,weight4.IS2)
    output.48weeks.x.percent.y.percent.half.sharing.bad[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                                c(0.01,0),primary.burden,secondary.burden,0.7,
                                                                                0.1,0.05,epsilon1.bad,0.7,0,rho1.bad,
                                                                                one.dose.burden.bad,two.doses.burden,vaccine.burden,0.5,
                                                                                45,0.06,60,c(y.percent/4,y.percent/4),1/4,
                                                                                weight.IS,weight.IS1,weight.IS2,
                                                                                weight2.IS,weight2.IS1,weight2.IS2,
                                                                                weight3.IS,weight3.IS1,weight3.IS2,
                                                                                weight4.IS,weight4.IS1,weight4.IS2)
    output.48weeks.x.percent.y.percent.no.sharing.good[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                               c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                               0.1,0.05,epsilon1.good,0.7,0,rho1.good,
                                                                               one.dose.burden.good,two.doses.burden,vaccine.burden,0.5,
                                                                               45,0.06,60,c(y.percent/2,0),1/4,
                                                                               weight.IS,weight.IS1,weight.IS2,
                                                                               weight2.IS,weight2.IS1,weight2.IS2,
                                                                               weight3.IS,weight3.IS1,weight3.IS2,
                                                                               weight4.IS,weight4.IS1,weight4.IS2)
    output.48weeks.x.percent.y.percent.half.sharing.good[[i]]=plots.doses.switch(1,7,35,48,80,1/(1*52),1/(1*52),48,
                                                                                 c(x.percent,0),primary.burden,secondary.burden,0.7,
                                                                                 0.1,0.05,epsilon1.good,0.7,0,rho1.good,
                                                                                 one.dose.burden.good,two.doses.burden,vaccine.burden,0.5,
                                                                                 45,0.06,60,c(y.percent/4,y.percent/4),1/4,
                                                                                 weight.IS,weight.IS1,weight.IS2,
                                                                                 weight2.IS,weight2.IS1,weight2.IS2,
                                                                                 weight3.IS,weight3.IS1,weight3.IS2,
                                                                                 weight4.IS,weight4.IS1,weight4.IS2)
  }
  panels=list()
  panels[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[1]]|
                  output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[1]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[1]]|
                                                                                                          output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[1]]))
  panels[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[1]]|
                  output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[1]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[1]]|
                                                                                     output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[1]]))
  panels.evo=list()
  panels.evo[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[2]]|
                      output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[2]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[2]]|
                                                                                                              output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[2]]))
  panels.evo[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[2]]|
                      output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[2]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[2]]|
                                                                                         output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[2]]))
  panels.evo.total=list()
  panels.evo.total[[1]]=((output.48weeks.x.percent.y.percent.no.sharing.bad[[1]][[3]]|
                            output.48weeks.x.percent.y.percent.half.sharing.bad[[1]][[3]]+xlab(element_blank()))/(output.48weeks.x.percent.y.percent.no.sharing.bad[[2]][[3]]|
                                                                                                                    output.48weeks.x.percent.y.percent.half.sharing.bad[[2]][[3]]))
  panels.evo.total[[2]]=((output.48weeks.x.percent.y.percent.no.sharing.good[[1]][[3]]|
                            output.48weeks.x.percent.y.percent.half.sharing.good[[1]][[3]])/(output.48weeks.x.percent.y.percent.no.sharing.good[[2]][[3]]|
                                                                                               output.48weeks.x.percent.y.percent.half.sharing.good[[2]][[3]]))
  return(list(panels,panels.evo,panels.evo.total))
}

weight.IS=0.05
weight.IS1=0.3
weight.IS2=0.05
weight2.IS=0.05
weight2.IS1=1
weight2.IS2=0.05
weight3.IS=0.8
weight3.IS1=1
weight3.IS2=0.8
weight4.IS=1
weight4.IS1=0.05
weight4.IS2=0.05

out1.output.all=x.percent.y.percent.switch.one.dose(c(0.01,0.01),c(0.03,0.05),
                                                1/(1*52),1/(0.25*52),
                                                0.7,0.9,
                                                weight.IS,weight.IS1,weight.IS2,
                                                weight2.IS,weight2.IS1,weight2.IS2,
                                                weight3.IS,weight3.IS1,weight3.IS2,
                                                weight4.IS,weight4.IS1,weight4.IS2)

out2.output.all=x.percent.y.percent.switch.one.dose.switch.two.dose(c(0.01,0.01),c(0.03,0.05),
                                                                1/(1*52),1/(0.25*52),
                                                                0.7,0.9,
                                                                weight.IS,weight.IS1,weight.IS2,
                                                                weight2.IS,weight2.IS1,weight2.IS2,
                                                                weight3.IS,weight3.IS1,weight3.IS2,
                                                                weight4.IS,weight4.IS1,weight4.IS2)


process.out=function(out1,out2){
  out1[[1]][[1]][[1]][[1]]=out1[[1]][[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[2]][[1]][[1]][[1]]=out1[[2]][[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[1]][[1]][[1]][[1]]=out2[[1]][[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))+theme(plot.margin=unit(c(0,0,0,0),"cm"))
  
  out2[[2]][[1]][[1]][[1]]=out2[[2]][[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[1]][[1]][[2]][[1]]=out1[[1]][[1]][[2]][[1]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[2]][[1]][[2]][[1]]=out1[[2]][[1]][[2]][[1]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[1]][[1]][[2]][[1]]=out2[[1]][[1]][[2]][[1]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[2]][[1]][[2]][[1]]=out2[[2]][[1]][[2]][[1]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  return(list(out1,out2))
}

process.out.evo.total=function(out1,out2){
  out1[[1]][[1]][[1]]=out1[[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[2]][[1]][[1]]=out1[[2]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[1]][[1]][[1]]=out2[[1]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))+theme(plot.margin=unit(c(0,0,0,0),"cm"))
  
  out2[[2]][[1]][[1]]=out2[[2]][[1]][[1]]+ggtitle(label=element_blank(),subtitle="No sharing (f=0)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[1]][[1]][[2]]=out1[[1]][[1]][[2]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out1[[2]][[1]][[2]]=out1[[2]][[1]][[2]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[1]][[1]][[2]]=out2[[1]][[1]][[2]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  
  out2[[2]][[1]][[2]]=out2[[2]][[1]][[2]]+ggtitle(label=element_blank(),subtitle="Equal sharing (f=0.5)")+
    theme(plot.subtitle=element_text(hjust=0.5))
  return(list(out1,out2))
}


generate.plot=function(input){
  out1.processed=input[[1]]
  out2.processed=input[[2]]
  
  panel1=wrap_elements(wrap_elements(out1.processed[[1]])+
                         plot_annotation(title="One-dose policy",
                                         subtitle=element_blank())&
                         theme(plot.margin=unit(c(0,0,0,0),"cm"),
                               plot.title=element_text(size=18)))
  panel2=wrap_elements(wrap_elements(out1.processed[[2]])+
                         plot_annotation(title="One-dose policy",
                                         subtitle=element_blank())&
                         theme(plot.margin=unit(c(0,0,0,0),"cm"),
                               plot.title=element_text(size=18)))
  
  panel3=wrap_elements(wrap_elements(out2.processed[[1]])+
                         plot_annotation(title="Switch to two-dose strategy",
                                         subtitle=element_blank())&
                         theme(plot.margin=unit(c(0,0,0,0),"cm"),
                               plot.title=element_text(size=18)))
  
  panel4=wrap_elements(wrap_elements(out2.processed[[2]])+
                         plot_annotation(title="Switch to two-dose strategy",
                                         subtitle=element_blank())&
                         theme(plot.margin=unit(c(0,0,0,0),"cm"),
                               plot.title=element_text(size=18)))
  
  top.row=wrap_elements(wrap_elements(panel1|panel3&theme(plot.margin=unit(c(0,0,0,0),"cm")))+
                          plot_annotation(title="Poor vaccinal immunity after one dose",
                                          theme=theme(plot.title=element_text(size=20))))&
    theme(plot.margin=unit(c(0,0,0,0),"cm"))
  
  bottom.row=wrap_elements(wrap_elements(panel2|panel4&theme(plot.margin=unit(c(0,0,0,0),"cm")))+
                             plot_annotation(title="Robust vaccinal immunity after one dose",
                                             theme=theme(plot.title=element_text(size=20))))&
    theme(plot.margin=unit(c(0,0,0,0),"cm"))
  
  combined=top.row/bottom.row
  return(combined)
}

out1.output.epi=out1.output.all[[1]]
out2.output.epi=out2.output.all[[1]]
out1.output.evo=out1.output.all[[2]]
out2.output.evo=out2.output.all[[2]]
out1.output.evo.total=out1.output.all[[3]]
out2.output.evo.total=out2.output.all[[3]]
processed.evo=process.out(out1.output.evo,out2.output.evo)
processed.evo.total=process.out.evo.total(out1.output.evo.total,out2.output.evo.total)
processed.epi=process.out(out1.output.epi,out2.output.epi)
combined.evo=generate.plot(processed.evo)
combined.evo.total=generate.plot(processed.evo.total)
combined.epi=generate.plot(processed.epi)

#pdf("combined-feb3-evo-feb11-feb14.pdf",height=18,width=14)
pdf("Fig-Extra-1-April16-May18-May26.pdf",height=18,width=14)
print(combined.evo)
dev.off()

#pdf("combined-feb3-evo-total-feb11-feb14.pdf",height=18,width=14)
pdf("Fig-Extra-2-April16-May18-May26.pdf",height=18,width=14)
print(combined.evo.total)
dev.off()

#pdf("combined-feb3-epi-feb11-feb14.pdf",height=18,width=14)
pdf("Fig3-April16-May18-May26.pdf",height=18,width=14)
print(combined.epi)
dev.off()


  

