library(ggplot2)
calculating.IT=function(nu,beta,gamma,mu,delta.v,delta,epsilon){
  Q=beta/(gamma+mu)
  A=(delta*gamma)/(delta+mu)
  B=(delta.v*nu)/(delta.v+mu)
  C=mu
  D=beta
  E=mu+nu
  G=epsilon*beta
  H=mu+nu-((delta.v*nu)/(delta.v+mu))
  
  M=epsilon*A*D-((G*D)/Q)
  N=G*C+epsilon*A*E-((G*E)/Q)-((D*H)/Q)
  P=C*H+epsilon*B*C-((H*E)/Q)
  root.1=((-N+sqrt(N^2-(4*M*P)))/(2*M))
  root.2=((-N-sqrt(N^2-(4*M*P)))/(2*M))
  IT.value=max(root.1,root.2)
  if (IT.value<0){
    IT.value=0
  }
  return(IT.value)
}

computing.total.IT.half=function(nu.total,beta.p,beta.d,delta.v,delta,epsilon,phiBA){
  mu=1/(50*365)
  gamma=1/5
  n.length=301
  f=seq(from=0,to=0.5,length.out=n.length)
  nu.p=nu.total*(1-f)
  nu.d=nu.total*(1/phiBA)*f
  IT.p=sapply(nu.p,FUN=calculating.IT,beta=beta.p,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.d=sapply(nu.d,FUN=calculating.IT,beta=beta.d,gamma=gamma,
              mu=mu,delta.v=delta.v,delta=delta,epsilon=epsilon)
  IT.sum=IT.p+(phiBA*IT.d)
  IT.avg=IT.sum/(1+phiBA)
  return(IT.avg)
}

  phiBA=1
  nu=0.004
  beta.p=2.3/5
  beta.d=2.3/5
  nu.IT.vector.intermediate=computing.total.IT.half(nu.total=nu,beta.p=beta.p,
                                   beta.d=beta.d,delta.v=1/(1*365),
                                   delta=1/(1*365),epsilon=0.7,phiBA=phiBA)
  nu.IT.vector.intermediate.increased.transmission=computing.total.IT.half(nu.total=nu,beta.p=1.3*beta.p,
                                  beta.d=1.3*beta.d,delta.v=1/(1*365),
                                  delta=1/(1*365),epsilon=0.7,phiBA=phiBA)
  nu.IT.vector.good=computing.total.IT.half(nu.total=nu,beta.p=beta.p,
                                               beta.d=beta.d,delta.v=1/(1.5*365),
                                               delta=1/(1.5*365),epsilon=0.6,phiBA=phiBA)
  nu.IT.vector.good.increased.transmission=computing.total.IT.half(nu.total=nu,beta.p=1.3*beta.p,
                                                                      beta.d=1.3*beta.d,delta.v=1/(1.5*365),
                                                                      delta=1/(1.5*365),epsilon=0.6,phiBA=phiBA)
  nu.vectors=c(rep(nu,length(nu.IT.vector.intermediate)),
               rep(nu,length(nu.IT.vector.intermediate.increased.transmission)),
               rep(nu,length(nu.IT.vector.good)),
               rep(nu,length(nu.IT.vector.good.increased.transmission)))
  fractions=c(seq(from=0,to=0.5,length.out=length(nu.IT.vector.intermediate)),
              seq(from=0,to=0.5,length.out=length(nu.IT.vector.intermediate)),
              seq(from=0,to=0.5,length.out=length(nu.IT.vector.intermediate)),
              seq(from=0,to=0.5,length.out=length(nu.IT.vector.intermediate)))
  IT.values=c(nu.IT.vector.intermediate,nu.IT.vector.intermediate.increased.transmission,
              nu.IT.vector.good,nu.IT.vector.good.increased.transmission)
  immunity.type.vector=c(rep("intermediate immunity",length(nu.vectors)/2),
                         rep("good immunity",length(nu.vectors)/2))
  immunity.type.vector.factor=factor(immunity.type.vector,levels=c("intermediate immunity","good immunity"))
  beta.type.vector=c(rep("overall lower transmission",length(nu.vectors)/4),
                     rep("overall higher transmission",length(nu.vectors)/4),
                     rep("overall lower transmission",length(nu.vectors)/4),
                     rep("overall higher transmission",length(nu.vectors)/4))
  output=data.frame("nu"=nu.vectors,"f"=fractions,"IT"=IT.values,
                    "immunity"=immunity.type.vector.factor,
                    "beta.type"=as.character(beta.type.vector),stringsAsFactors=FALSE)
  f=ggplot(data=output)+geom_line(aes(x=f,y=IT,color=beta.type),size=0.8)+facet_grid(~immunity)+
    theme(panel.spacing=unit(5,"lines"))+theme_minimal()+scale_x_continuous(breaks=c(0,0.5),
                                                                         labels=c("stockpiling","equity"))+
    xlab("vaccine allocation strategy")+
    ylab("fraction infected")+
    theme(panel.spacing=unit(2,"lines"))+
    scale_color_manual(name="",values=c("red","blue"))+
    theme(legend.position=c(0.8,0.75))+
    theme(panel.grid.minor=element_blank(),
          strip.text=element_text(size=11,face="bold"))
  pdf("Inset_July7.pdf",width=6,height=2.4)
  print(f)
  dev.off()
  
